\\ Copyright 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

allocatemem(100*10^6);
default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

factor_list(n) =
{
  my(v=Vec(factor(n)~));
  if(#v==0, [n], concat(apply(v->vector(v[2],i,v[1]),v)));
}

\\-----------------------------------------------------------------------------

k=8;
den=15;
m = fractional_location_map(den,k);
m=Mat(m);
m=Vec(m[,2]);
m=select(v->#v==3,m);
m=apply(Vec,m);
m=concat(m);
my(limit=den*BlobN(k)); m=select(n->n<=limit, m);
m=vecsort(m);
\\m

round_up_multiple(n,d) = ceil(n/d)*d;
round_down_multiple(n,d) = floor(n/d)*d;

print("across even");
{
  my(d=2*den);
  my(seen_diffs=Set([]));
  my(seen_pairs=Set([]));
  for(i=1,#m-1,
     my(a=m[i], b=m[i+1]);
     my(n=ceil(a/d));
     my(p=d*n);
     my(aa=p-a, bb=b-p);
     if(! (0 < aa && aa < 15 && 0 < bb && bb < 15), next());
     my(double=if(DpredInf(n),"double",""));
     my(diff=aa+bb);
     my(pair=Str(aa" to "bb));
     \\ print(a"  "p"  "b"  "pair" = "diff"  "double,
     \\       if(setsearch(seen_diffs,diff)==0,"  ***",""));
     seen_diffs=setunion(seen_diffs,[diff]);
     pair=Str(min(aa,bb)," to ",max(aa,bb));
     seen_pairs=setunion(seen_pairs,[pair]);
  );
  print("seen "seen_diffs"\n"seen_pairs);
}
     
print("across odd");
{
  my(d=2*den);
  my(seen_diffs=Set([]));
  my(seen_pairs=Set([]));
  for(i=1,#m-1,
     my(a=m[i], b=m[i+1]);
     my(n=floor(a/d));
     my(p=d*n + 15);
     my(aa=p-a, bb=b-p);
     if(! (0 < aa && aa < 15 && 0 < bb && bb < 15), next());
     my(double=if(DpredInf(n),"double",""));
     my(diff=aa+bb);
     my(pair=Str(aa" to "bb));
     \\ print(a"  "p"  "b"  "pair" = "diff"  "double,
     \\       if(setsearch(seen_diffs,diff)==0,"  ***",""));
     seen_diffs=setunion(seen_diffs,[diff]);
     pair=Str(min(aa,bb)," to ",max(aa,bb));
     seen_pairs=setunion(seen_pairs,[pair]);
  );
  print("seen "seen_diffs"\n"seen_pairs);
}



print("within segment");
{
  my(d=2*den);
  my(seen_diffs=Set([]));
  my(seen_pairs=Set([]));
  for(i=1,#m-1,
     my(a=m[i], b=m[i+1]);
     my(n_lo=a\den);
     my(p=den*n_lo);
     my(aa,bb);
     if(n%2, p+=15; aa=p-b; bb=p-a,
             aa=a-p; bb=b-p);
     \\ print(aa"  "bb);
     if(! (0 < aa && aa < 15 && 0 < bb && bb < 15), next());
     
     my(n_hi=n_lo+1);
     my(pre_double=if(DpredInf(n_lo),"D","S"));
     my(post_double=if(DpredInf(n_hi),"D","S"));

     my(bridge="");
     for(j=4,k,
        my(bn=BlobNend(j));
        for(l=0,2,
           if(n_lo==bn+l,bridge=Str(" bridge+"l" after "j);break(2))));

     my(diff=bb-aa);
     my(pair=Str(aa" to "bb));
     print("n="n_lo" p="p"  "a" "b"  "pair" = "diff"  "
           pre_double"-"post_double,
           bridge,
           if(setsearch(seen_diffs,diff)==0,"  ***",""));
     seen_diffs=setunion(seen_diffs,[diff]);
     pair=[aa,bb];
     seen_pairs=setunion(seen_pairs,[pair]);
  );
  print("seen "seen_diffs"\n"seen_pairs);
}
     
