\\ Copyright 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

allocatemem(100*10^6);
default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

factor_list(n) =
{
  my(v=Vec(factor(n)~));
  if(#v==0, [n], concat(apply(v->vector(v[2],i,v[1]),v)));
}

\\-----------------------------------------------------------------------------

fracper_to_frac([ [1]],b)
fracper_to_frac([1, [1]],b)

\\       v   v   v  sum = 1/(B-1)
\\   . xxx xxx xxx
\\ 0  -1 -3  -6  -9
\\ sum(j=0,k, b^j) = (b^(k+1) - 1) / (b - 1)
\\ sum(j=1,k, b^j) = (b^(k+1) - 1) / (b - 1) - 1
\\ sum(j=1,k, b^j) = b * (b^k - 1) / (b - 1)
\\ sum(j=1,k, (1/b)^j) = 1/b * ((1/b)^k - 1) / (1/b - 1)
\\ sum(j=1,k, (1/b)^j) = ((1/b)^k - 1) / (1 - b)
\\                    -> 1 / (b - 1)
sum(k=1,100,b^(-1*k))*1.0
1/(b-1)

\\ quit

\\-----------------------------------------------------------------------------

print();
print("f=0");
f=0
v = frac_to_fracper(f)
fracper_to_frac(v,b)
dragon_point_by_sum(floor(f*2^16))

print();
print("f=1");
f=1;
v = frac_to_fracper(f)
\\ fracper_to_frac(v,b)
\\ dragon_point_by_sum(floor(f*2^16))*-I
check_exact( 1/b + 1/b*(1/(b-1)), -I);
\\ n=256 p=b^8
\\ n=255 p=b^8-I
\\ n=0.1111_1111 p=1-I/b^8
check_exact(b^8, 16, "b^8")
check_exact(sum(i=0,6,-b^i), 8+7*I)
check_exact(b^7+sum(i=0,6,-b^i), 16-I)
check_exact(fracper_to_frac([1,-1,-1,-1,-1,-1,-1,-1, []],b), 1-1/16*I);
check_exact(fracper_to_frac([1, [-1]],b), 1, "1 then -1s");

\\-----------------------------------------------------------------------------

print();
f=13/30
v = frac_to_fracper(f)
fracper_to_frac(v)
fracper_to_frac(v,b)
dragon_point_by_sum(floor(f*2^16))
for(k=0,8,print((1/2 - 1/6*I)*b^k));


check_exact(dragon_fpoint(3/16+5/7/16), 3/20 - 9/20*I)
check_exact(dragon_fpoint(4/16+6/7/16), 3/20 - 9/20*I)
check_exact(dragon_fpoint(4/16+10/7/16), 3/20 - 9/20*I)
check_exact(3/16+ 5/7/16, 13/(7*2^3))
check_exact(4/16+ 6/7/16, 17/(7*2^3))
check_exact(4/16+10/7/16, 19/(7*2^3))

den_count_triples(den) =
{
  my(k=6,m=Map());
  den *= 2^k;
  for(num=1,den-1,
     my(z=dragon_fpoint(num/den));
     mapput(m,z, 1+if(mapisdefined(m,z), mapget(m,z))));
  m=Mat(m);
  my(counts=vector(10));
  for(i=1,matsize(m)[1],
     my(count=m[i,2]);
     if(count>1 && count <=#counts,
        counts[count]++));
  while(#counts > 1 && counts[#counts]==0, counts=counts[1..#counts-1]);
  counts;
}

\\ multiples of 7 or 15
{
  forstep(den=3,100, 2,
         my(f=factor_list(den));
         my(counts=den_count_triples(den));
         print("den ",den,
               "  count ",counts,
               "  ",f,
               if(#counts>=3, "    ******","")));
}
quit

\\-----------------------------------------------------------------------------
{
  \\ triples   7,15,          255,511
  \\ not     3,     31,63,127,       2047
  my(den=17);
  print("den "den);
  my(k=6,m=Map());
  den *= 2^k;
  for(num=1,den-1,
     my(z=dragon_fpoint(num/den));
     my(l=if(mapisdefined(m,z),
             l=mapget(m,z),
             List([])));
     listput(l,num);
     mapput(m,z,l));
  m=Mat(m);
  for(i=1,matsize(m)[1],
     my(z=m[i,1],l=m[i,2]);
     if(#l==3,
        print(z"  "Vec(l)"   "Vec(l)/den));
  );
  my(v=m[,2]);
  v=select(x->#x==3,v);
  \\ v=apply(x->numerator(x[1]/den),v);
  v=apply(x->x[1],v);
  v=Vec(Set(v));
  print(vecsort(v));
  
}
quit

print();
f=1/2
v = frac_to_fracper(f)
v = fracper_to_frac(v)
fracper_to_frac(v,b)
dragon_point_by_sum(floor(f*2^16))

print();
f=1/3
v = frac_to_fracper(f)
v = fracper_to_frac(v)
fracper_to_frac(v,b)
dragon_point_by_sum(floor(f*2^16))

print();
f=47/60
v = frac_to_fracper(f)
v = fracper_to_frac(v)
fracper_to_frac(v,b)
dragon_point_by_sum(floor(f*2^16))
check_exact(fracper_to_frac(v,b), conj(2/3 + I/6))
\\
f=47/120;
v = frac_to_fracper(f);
v = fracper_to_frac(v);
check_exact(fracper_to_frac(v,b), conj(1/4 + (5*I)/12))
f=47/240;
v = frac_to_fracper(f);
v = fracper_to_frac(v);
check_exact(fracper_to_frac(v,b), conj(-1/12 + I/3))

print();
f=13/15/4
v = frac_to_fracper(f)
v = fracper_to_frac(v)
fracper_to_frac(v,b)
dragon_point_by_sum(floor(f*2^16))
f=21/15/4
v = frac_to_fracper(f)
v = fracper_to_frac(v)
fracper_to_frac(v,b)
dragon_point_by_sum(floor(f*2^16))
f=23/15/4
v = frac_to_fracper(f)
v = fracper_to_frac(v)
fracper_to_frac(v,b)
dragon_point_by_sum(floor(f*2^16))
quit

\\ lg(23/15) = 1  factor >=2
{
  my(seen=Map());
  for(n=1,#A260748,
     fif=[A260748[n], A260749[n], A260750[n]];
     if(gcd(fif)>1,next());
  k=floor( log(vecmax(fif)/15)/log(2) )+1;
     k=6;
  print("\n",fif" k="k);
  for(i=1,#fif,
     f=fif[i]/15 /2^k;
     print(dragon_fpoint(f),"   ",dragon_fpoint(f)*2^ceil(k/2)*8);
  );
);
}

print();
{
  for(n=1,100,
     my(f=n/15 / 256);
     printf("%3d  ",n);
     print(dragon_fpoint(f));
  );
}
quit

