\\ Copyright 2015, 2017, 2018 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(strictargs,1);
b = 1+I;
vector_modulo(v,i) = v[(i% #v)+1];
gf_terms(g,n) = my(x=variable(g)); \
  concat(vector(min(n,valuation(g,x)), i, 0), \
         Vec(g + O(x^n)));

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
\\ Gray code endpoint

\\      *
\\      |  k=1
\\      v  end=1
\\ *--->

\\ *
\\ |       k=2
\\ v       end=I
\\  <---*
\\      |
\\      v
\\ *--->

midpoint_right(start,end) = (start+end)/2 + (-I)*(end-start)/2;
check_exact(midpoint_right(0,1+I), 1)
check_exact(midpoint_right(2*I, 1+I), I)

Gray_end(k, start, end) =
{
  if(k<=1, end,
     my(m = midpoint_right(start,end));
     Gray_end(k-1, end,m));
}
vector(14,k,my(k=k-1); Gray_end(k,0,b^k))

Gray_end_bk(k) =
{
  if(k==0, 1,
     k==1, 1+I,
     b^k + (-I)*Gray_end_bk(k-1));
}
check_exact(vector(10,k,my(k=k-1); Gray_end_bk(k)), \
            vector(10,k,my(k=k-1); Gray_end(k,0,b^k)));

Gray_end_bk(k) =
{
  if(k==0, 1,
     (-I)^(k-1)*(1+I) + sum(j=0,k-2, (-I)^j*b^(k-j)));
}
check_exact(vector(10,k,my(k=k-1); Gray_end_bk(k)), \
            vector(10,k,my(k=k-1); Gray_end(k,0,b^k)));

Gray_end_bk(k) =
{
  if(k==0, 1,
    (-1 + I) *(-I)^(k) + sum(j=0,k-2, (-I)^j*b^(k-j)));
}
check_exact(vector(10,k,my(k=k-1); Gray_end_bk(k)), \
            vector(10,k,my(k=k-1); Gray_end(k,0,b^k)));

Gray_end_bk(k) =
{
  if(k==0, 1,
    (-1 + I) *(-I)^(k) + b^2* ((-I)^(k-1) - b^(k-1)) / (-I - b));
}
check_exact(vector(10,k,my(k=k-1); Gray_end_bk(k)), \
            vector(10,k,my(k=k-1); Gray_end(k,0,b^k)));

Gray_end_bk(k) =
{
  if(k==0, 1,
    (-1 + I) *(-I)^(k) +  ((-4/5 - 2/5*I)*(-I)^(k-1) - (-4/5 - 2/5*I)*b^(k-1)));
}
check_exact(vector(10,k,my(k=k-1); Gray_end_bk(k)), \
            vector(10,k,my(k=k-1); Gray_end(k,0,b^k)));

Gray_end_bk(k) =
{
  if(k==0, 1,
    1/5*(3-I)*b^k + (-3+I)/5*(-I)^k );
}
check_exact(vector(10,k,my(k=k-1); Gray_end_bk(k)), \
            vector(10,k,my(k=k-1); Gray_end(k,0,b^k)));


\\-----------------------------------------------------------------------------
\\ pen up by drawing

draw(k) =
{
  pen_location = 0;
  move_total = 0;
  draw_recurse(k, 0, b^k);
  move_total;
}
draw_recurse(k, start, end) =
{
  if(k<=1,
     move(start);
     pen_location=end,
     \\
     my(m = midpoint_right(start,end));
     draw_recurse(k-1, start, m);
     draw_recurse(k-1, end, m));
}
draw_line(start,end) =
{
  pen_state = 0;
  move(start);
  pen_location = end;
}
move(to) =
{
  move_total += abs(to - pen_location);
  pen_location = to;
}
vector(14,k,my(k=k-1); draw(k))

max_real_imag(z) =max(abs(real(z)),abs(imag(z)));


g_Gray_step_max_real_imag(x) = 1/5*( (6 + 8*x)/(1-2*x^2)  - (1 + 3*x - 3*x^2 + x^3 - x^4*(1 +3*x - 3*x^2 + x^3))/(1-x^8));
check_exact(5*(gf_terms(g_Gray_step_max_real_imag(x),20) - \
               vector(20,k,my(k=k-1); max_real_imag(b^(k+1) - Gray_end_bk(k)))), \
            vector(20,k,my(k=k-1); 0));

pen_up(k) =
{
  if(k<=1,0,  2*pen_up(k-1) + max_real_imag(b^k - Gray_end_bk(k-1)));
}
print("pen_up move max ",vector(16,k,my(k=k-1); pen_up(k)))

\\ -> 2.1*2^k of hypotenuse distance
\\ -> 1.941176*2^k of longer re,im   16/17
my(k=500); pen_up(k)/2^k*1.0 
\\ my(k=20000); pen_up(k)/2^k*1.0 
print("33/17 = ",33/17.0);

pen_up_ref(k) = if(k<=1, 0,  2*pen_up_ref(k-1) + max_real_imag(b^k - Gray_end_bk(k-1)));

print("Gray_end_bk ",vector(10,k,my(k=k-1); Gray_end_bk(k)));
print("move delta ",vector(10,k,my(k=k-1);  (b^k - Gray_end_bk(k-1))));
print("g_Gray_step_max_real_imag ",gf_terms(g_Gray_step_max_real_imag(x),10));
print("pen_up_ref ",vector(10,k,my(k=k-1);  pen_up_ref(k)));

g_pen_up(x) = x/(1-2*x) *(g_Gray_step_max_real_imag(x) - 1);
check_exact(5*(gf_terms(g_pen_up(x),20) - \
               vector(20,k,my(k=k-1); pen_up_ref(k))), \
            vector(20,k,my(k=k-1); 0));



g_pen_up(x) = 1/2  + 45/34*1/(1-2*x) + (14*x + 10)/(10*x^2 - 5) + (x^3 - 25*x^2 + 13*x + 15)/(85*x^4 + 85);
check_exact(5*(gf_terms(g_pen_up(x),20) - \
               vector(20,k,my(k=k-1); pen_up_ref(k))), \
            vector(20,k,my(k=k-1); 0));

pen_up(k) =
{
  if(k==0,return(0));
  (45/34*2^k
        - vector_modulo( [ 2, 14/5 ],k)*2^floor(k/2)
        + vector_modulo([3/17, 13/85, -5/17, 1/85, -3/17, -13/85, 5/17, -1/85],k));
}
check_exact(17*(vector(10,k,my(k=k-1); pen_up(k)) - \
                vector(10,k,my(k=k-1); pen_up_ref(k))), \
            vector(10,k,my(k=k-1); 0));

