#!/usr/bin/perl -w

# Copyright 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use Carp 'croak';
use File::Spec;
use List::Util 'max','min','sum';
use Math::BaseCnv 'cnv';

use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()
use Math::PlanePath::Base::Digits
  'bit_split_lowtohigh', 'digit_split_lowtohigh',
  'digit_join_lowtohigh', 'round_down_pow';
use Math::PlanePath::Base::Generic
  'round_nearest';

# use Math::PlanePath;
# *_divrem_mutate = \&Math::PlanePath::_divrem_mutate;
# unshift @INC, '/usr/share/perl5';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyDragon ':all';
our @ipc;
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;


{
  # Pagoda K mod 3
  # looks at some low bits, then bit above

  run_foma(
           "source tools/dragon.foma",
           "regex [[Pagoda0.r] : z ] 
                | [[Pagoda1.r] : 1 ]
                | [[Pagoda2.r] : 2 ];",

           "regex [ [ [Pagoda0] -> z || _ .#. ]
                  | [ [Pagoda1] -> 1 || _ .#. ]
                  | [ [Pagoda2] -> 2 || _ .#. ] ].r;",

           "regex [ [ [Pagoda0.r] -> a || .#. _ ]
                  | [ [Pagoda1.r] -> b || .#. _ ]
                  | [ [Pagoda2.r] -> c || .#. _ ] ];",

           "regex [ [ [Pagoda0.r] -> a || .#. _ ]
                  | [ [Pagoda1.r] -> b || .#. _ ]
                  | [ [Pagoda2.r] -> c || .#. _ ] ];",

           "write att /tmp/Pagoda.att",
           "print dot >/tmp/Pagoda.dot",
          );
  require MyGraphs;
  MyGraphs::graphviz_view_file('/tmp/Pagoda.dot');
  exit 0;

  sub run_foma {
    my $str = join("\n",@_) . "\n";
    require File::Slurp;
    require IPC::Run;
    File::Slurp::write_file('/tmp/tempfile.foma',$str);
    IPC::Run::run(['foma','-p','-f','/tmp/tempfile.foma'],
                  '<','/dev/null') or die;
  }
  exit 0;
}
