\\ Copyright 2014, 2015, 2017, 2020 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");

L(k) = if(k<0,error("L k<0"), k<=3,2^k, L(k-1) + 2*L(k-3));
L=memoize(L);
gL(x) = -1 + 2* (1 + x^2) / (1 - x - 2*x^3);

R(k)=1 + sum(j=0,k-1, L(j));
R = memoize(R);
gR(x)= 1 - 2/(1-x) + 2*(1 + x + x^2)/(1-x-2*x^3);

B(k) = R(k+1);
gB(x) = 2*( - 1/(1-x) + (2 + x + 2*x^2)/(1-x-2*x^3));

A(k) = 2^(k-1) - B(k)/4;
AL(k) = if(k==0,return(0)); A(k) - A(k-1);
AR(k) = if(k==0,0, A(k-1));
gA(x)=x^4 / ((1 - x - 2*x^3)*(1-x)*(1-2*x));
gAL(x)=x^4 / ((1 - x - 2*x^3)*(1-2*x));
gAR(x) = x*gA(x);

JA(k) = A(k+1) - 2*A(k);

\\            bit 0 1
Rpred_matrix = {[ 0,1;    /* rs0 */
                  2,3;    /* rs1 */
                  0,5;    /* rs2 */
                  4,3;    /* rs3 */
                  8,5;    /* rs4 */
                  6,7;    /* rs5 */
                  2,9;    /* rs6 */
                  9,9;    /* rs7 */
                  9,3;    /* rs8 */
                  9,9 ]}; /* non */
Rpred_transition(state,bit) = Rpred_matrix[state+1,bit+1];
Rpred_transition_run(s,v) = for(i=1,#v, s=Rpred_transition(s,v[i])); s;
Rpred_state_by_high_to_low(n) = Rpred_transition_run(0,binary(n));
Rpred(n) = Rpred_state_by_high_to_low(n) != 9;
Rpred_v(v) = Rpred_transition_run(0,v) != 9;

unbinary(v) = subst(Pol(v),'x,2);


\\-----------------------------------------------------------------------------

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_count = 0;
check_equal(got,want) =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok "check_count));
}

check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong  ",name,"\n");
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count));
}

gf_terms(g,n) = my(x=variable(g)); \
  concat(vector(min(n,valuation(g,x)), i, 0), \
         Vec(g + O(x^n)));

binary_padded(n,k) =  \
  my(v = binary(n)); \
  concat(vector(max(0,k-#v)),v);

\\-----------------------------------------------------------------------------

bits_vector = vector(4,b,binary_padded(b-1,2));

R_by_Rpred(k) = sum(n=0,2^k-1,Rpred(n));
check_exact(vector(14,k,my(k=k-1); R_by_Rpred(k)), \
            vector(14,k,my(k=k-1); R(k)));

Rn_triplet_mod   = [ 1, 0; 2, 2 ];
Rn_triplet(len,bits) = \
  if(len<3,error("Rn_triplet len=",len" is < 3")); \
  my(mod = Rn_triplet_mod[bits[#bits-1]+1,bits[#bits]+1], \
     v = vector(len,i,(len-i+mod)%3==0)); \
  v[1] = 1-v[1]; \
  v[#v-1] = bits[#bits-1]; \
  v[#v]   = bits[#bits]; \
  v;

Rpred_locate_lowest_triplet(v) = {
  my(pos=0);
  for(i=1,#v-2,
      if(v[i]==0 && v[i+1]==0 && v[i+2]==0, pos=i);
      if(v[i]==1 && v[i+1]==0 && v[i+2]==1, pos=i);
      if(v[i]==1 && v[i+1]==1 && v[i+2]==0, pos=i);
      if(v[i]==1 && v[i+1]==1 && v[i+2]==1, pos=i));
  pos;
}

{
  for(len=4,7,
     for(b=1,4,
        my(v = Rn_triplet(len,bits_vector[b]));
        print(v,"  ",Rpred_locate_lowest_triplet(v)));
     print());
}

{
  for(len=3,10,
     for(b=1,4,
        my(bits = bits_vector[b],
           v = Rn_triplet(len,bits),
           pos = Rpred_locate_lowest_triplet(v));
        check_exact(pos,1, Str(v," bits=",bits))));
}

Rn_run(len,bit) = my(v=vector(len,i,bit)); v[1]=0; v[2]=1-v[2]; v;
lens_to_Rn(lens,bits) =
{
  my(t = Rn_triplet(lens[#lens],bits));
  lens[#lens]--;
  bits[#bits-1] = t[1];
  concat(concat(vector(#lens-1,i, Rn_run(lens[i],bits[i]))),
         t);
}

make_lens(k) =
{
  if(k<3,return([]));
  my(ret=List([]));
  for(first=3,k,
     my(lens_list=make_lens(k-first));
     \\ print("lens ",lens);
     for(i=1,#lens_list,
        my(lens=lens_list[i]);
        \\ print("sub-lens ",lens[i],"  put ",concat(first,lens));
        listput(ret,concat(first,lens))));
  listput(ret,[k]);
  Vec(ret);
}
make_lens = memoize(make_lens);

make_lens_2(k) = select((lens)->length(lens)>=2, make_lens(k));
make_lens_2 = memoize(make_lens_2);

for(k=0,9,print(k,"  ",make_lens_2(k)));

lens_to_v(lens,bits) =
{
  my(t = Rn_triplet(lens[#lens],bits));
  bits[#bits-1] = t[1];
  lens[#lens-1]--;
  my(v = concat(concat(vector(#lens-1,i, Rn_run(lens[i],bits[i]))),
                t));
  v[1] = 1;
  v;
}

{
  my(limit=6);
  my(seen=vector(2^(limit+1),i,0), count=0);
  for(k=0,limit,
     print("k="k);
     my(lens_list=make_lens_2(k+1));
     for(i=1,#lens_list,
        my(lens=lens_list[i]);
        for(b=0,2^#lens-1,
           my(bits=binary_padded(b,#lens));
           my(v=lens_to_v(lens,bits));
           seen[unbinary(v)] = 1;
           my(rpred=Rpred_v(v));
           my(pos = Rpred_locate_lowest_triplet(v));
           print(v,"  rpred=",rpred,"  pos=",pos," lens=",lens);
           count++));
     print("count="count", 4*AR("k")=",4*AR(k)," seen=",vecsum(seen));

     \\ print("cf Rpred");
     \\ for(n=0,2^k-1,
     \\    my(v=binary(n));
     \\    my(rpred=Rpred_v(v));
     \\    if(rpred,next());
     \\    my(pos = Rpred_locate_lowest_triplet(v));
     \\    print(v,"  ",rpred,"  pos=",pos,"  seen=",seen[n]));

     for(n=1,2^k-1,seen[n+2^k]=seen[n]);
     count *= 2;
  );
}

\\------------------------------------------------------------------------------

\\ of alternating 000 111 000
\\
\\ runs_ge3_list(k) = \
\\   if(k<3,return([])); \
\\   if(k==3,return([ [0,0,0], [1,1,1] ])); \
\\   concat(apply((e)->concat(e,[e[#e]]), runs_ge3_list(k-1)), \
\\          apply((e)->concat(e,[1-e[#e],1-e[#e],1-e[#e]]), runs_ge3_list(k-3)));
\\ runs_ge3_by_list(k) = length(runs_ge3_list(k));
\\ \\ for(k=0,8, my(v=runs_ge3_list(k));for(i=1,#v,print(v[i])))
\\ 
\\ \\ return the number of runs in vector v, which are >=3 long
\\ \\ or return 0 if not all are >=3
\\ vector_count_runs_ge3_all(v) =                  \
\\   if(v==[],return(0));                          \
\\   v[1]; /* must be vector */                    \
\\   my(run=1,count=1);                            \
\\   for(i=2,#v,                                   \
\\       if(v[i]==v[i-1],run++,  /* continues */   \
\\          run<3,return(0),     /* short */       \
\\          run=1;count++));     /* new */         \
\\   if(run<3,return(0));                          \
\\   count;

\\------------------------------------------------------------------------------

runs_ge3_list(k) = \
  if(k<3,return([])); \
  if(k==3,return([ [1,0,0] ])); \
  concat(apply((e)->concat(e,[0]), runs_ge3_list(k-1)), \
         apply((e)->concat(e,[1,0,0]), runs_ge3_list(k-3)));
runs_ge3_list = memoize(runs_ge3_list);
runs_ge3_by_list(k) = length(runs_ge3_list(k));
\\ for(k=0,8, my(v=runs_ge3_list(k));for(i=1,#v,print(v[i])))

\\ return the number of runs 10000 in vector v
\\ or return 0 if not all are >=3
vector_is_runs_ge3_count(v) =
{
  if(v==[],return(0));
  if(v[1]!=1,return(0));
  my(prev=1,count=1);
  for(i=2,#v,
      if(v[i],
         if(i-prev<3,return(0));  \\ short run
         prev=i));
  if(#v+1-prev<3,return(0));
  count;
}
for(n=1,2^8,my(v=binary(n),count=vector_is_runs_ge3_count(v));if(count,print(n,"  ",v)));
for(n=1,2^8,my(v=binary(n),count=vector_is_runs_ge3_count(v));if(count,print1(n,",")));print();
\\ A048715  bits spaced >=3 apart

runs_ge3_by_count(k) = \
  sum(i=0,2^k-1, vector_is_runs_ge3_count(binary_padded(i,k)) > 0);
print("runs_ge3_by_count ",vector(14,k,my(k=k-1); runs_ge3_by_count(k)))
check_exact(vector(14,k,my(k=k-1); runs_ge3_by_list(k)), \
            vector(14,k,my(k=k-1); runs_ge3_by_count(k)), \
            "list vs count");

\\ number of ways to make runs of >=3 bits within total k bits
runs_ge3_by_recurrence(k) =
{
  if (k<3, return(0));
  if (k==3, return(1));
  runs_ge3_by_recurrence(k-1)       \\ last run extends
  + runs_ge3_by_recurrence(k-3);  \\ new run of 3
}
runs_ge3_by_recurrence = memoize(runs_ge3_by_recurrence);
check_exact(vector(14,k,my(k=k-1); runs_ge3_by_recurrence(k)), \
            vector(14,k,my(k=k-1); runs_ge3_by_count(k)), \
            "recurrence vs count")

\\ wrong
\\ binomial_or_zero(n,m) = if(n<0,0, binomial(n,m));
\\ runs_ge3_by_binomial2(k) = \
\\   sum(b=1,k, binomial_or_zero(k-3*b,b)*b!);
\\ check_exact(vector(14,k,my(k=k-1); runs_ge3_by_binomial2(k)), \
\\             vector(14,k,my(k=k-1); runs_ge3_by_count(k)), \
\\             "binomial2 vs count")

runs_ge3_by_binomial(k) = \
  sum(t=0,k, my(n = k-3-2*t); if(n<t,0, binomial(n,t)));
check_exact(vector(14,k,my(k=k-1); runs_ge3_by_binomial(k)), \
            vector(14,k,my(k=k-1); runs_ge3_by_count(k)), \
            "binomial vs count")

\\------------------------------------------------------------------------------

Rnon_one(k) =
{
  if(k<3,return(0));
  if (k==3, return(2));
  return(
     Rnon_one(k-1)      \\ extend last run
     + 2*Rnon_one(k-3)  \\ new run in 2 types
  );
}
print("Rnon_one ",vector(14,k,Rnon_one(k-1)/2));
\\ = LQ2 = A052537
\\ = 2*dJA = 2*A003229 

Rnon_by_runs_exactly(k) =
{
  \\ two or more runs, l=first run length, k+1-l=further lengths
  return(sum(l=3,k,2*Rnon_one(k+1-l)));

  \\ t+1 many runs in k+1 bits (as triplet overlap)
  \\ first run is at top taking 3 bits
  \\ 2 bits for each of t are reserved
  \\ remaining k-3+1-2*t candidate positions for t runs
  return(  \\ works
  sum(t=1,k,
        my(n=k-3+1 - 2*t);
        if(n<t, 0,
           2^(t+1)*binomial(n,t)))
  );

  k = k-2;
  return(  \\ works
  2*sum(t=1,k,
        my(n=k-2*t);
        if(n<t, 0,
           2^t*binomial(n,t)))
  );

  \\ print("k=",k);
  return(
  sum(t=2,k,
        my(n=t);
        my(m=k-3*t);
        if(n<m, 0,
           \\ print("binomial ",n," ",t," = ",binomial(n,t));
           2^t*binomial(n,m)))
  );
}
Rnon_by_runs_exactly = memoize(Rnon_by_runs_exactly);
\\ = A003479 = JA

Rnon_by_runs(k) = \
  sum(l=5,k, 2^(k-l)*Rnon_by_runs_exactly(l));

print("AR   ",vector(15,k,my(k=k-1); 4*AR(k)))
print("Rnon ",vector(15,k,my(k=k-1); Rnon_by_runs(k)))
print("Rex  ",vector(20,k,my(k=k-1); Rnon_by_runs_exactly(k)))

check_exact(vector(15,k,my(k=k-1); Rnon_by_runs(k)), \
            vector(15,k,my(k=k-1); 4*AR(k)), \
            "Rnon_by_runs vs Rnon=4*AR")
