\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

gen_twindragon_area_tree_terminal_wiener_TAtwS_samples = [0,1,3,12,33,101,311];
gen_twindragon_area_tree_terminal_wiener_TATW_samples = [0,1,3,22,93,459,2423];

\\-----------------------------------------------------------------------------

\\ TDegCount1(k) = 2^(k-2) + 2   for k>=3

\\ k=1                    TAtwS(1) = 1      TATW(1) = 1
\\                        TAtwC(1) = 1
\\ 00--01                 TAtwSC(1) = 2

\\ k=2       10--11       TAtwS(2) = 3      TATW(2) = 3
\\            |             
\\       00--01             

\\ k=3       110--111     TAtwS(3) = 3+4+5 = 12
\\            |           TAtwC(3) = 3+3+4 = 10
\\      100--101          TATW(3) = 3+4+5 + 3+3+4 = 22
\\            |
\\           010--011
\\            |
\\      000--001     

\\ k=4      1110--1111               TAtwS(4) = 
\\            |                      TAtwC(4) = 
\\    1100--1101                     TATW(4) = 2*(22-10) + 2*3*10 + 3^2
\\            |                               
\\          1010--1011  0110--0111           
\\            |     |     |     
\\    1000--1001  0100--0101     
\\                        |
\\                      0010--0011
\\                        |
\\                0000--0001     

TAtwS(k) = {
  if(k<=3,return(gen_twindragon_area_tree_terminal_wiener_TAtwS_samples[k+1]));
   TAtwS(k-1)                        /* lower */
   - (RQ(k-1)-1)                     /* S to c1 excluded */
   + (TDegCount1(k-1)-1) * RQ(k-1)   /* S to c2 repeats */
   + TAtwC(k-1);                     /* c2 to upper */      
}
TAtwS = memoize(TAtwS);

TAtwC(k) = {
  if(k<3,error("TAtwC(k) is for k>=3"));
  if(k==3,return(10));             
  TAtwS(k-1)                         /* lower */            
  - (LQ(k-1)-1)                      /* S to c1 excluded */ 
  + (TDegCount1(k-1)-1) * LQ(k-1)    /* S to c2 repeats */  
  + TAtwC(k-1);                      /* c2 to upper */      
}
TAtwC = memoize(TAtwC);

\\-----------------------------------------------------------------------------
TAtwSC(k) = TAtwS(k) + TAtwC(k);

print("TAtwS  k>=3  ", vector(10,k,k+=2; TAtwS(k)));
print("TAtwC  k>=3  ", vector(10,k,k+=2; TAtwC(k)));
print("TAtwSC k>=3  ", vector(10,k,k+=2; TAtwSC(k)));

{
  check_exact(vector(#gen_twindragon_area_tree_terminal_wiener_TAtwS_samples,k,k--;
                     TAtwS(k)),
              gen_twindragon_area_tree_terminal_wiener_TAtwS_samples);   /* k>=3 */
}

TAtwSC_by_recurrence(k) =
{
  if(k==0,return(0)); \
  2*TAtwSC(k-1)
  - (BQ(k-1) - 2)
  + (2^(k-3)+1)*BQ(k-1);
}
check_exact(vector(100,k,k+=3; TAtwSC_by_recurrence(k)), \
            vector(100,k,k+=3; TAtwSC(k)), \
            "TAtwSC_by_recurrence 1");

TAtwSC_by_recurrence(k) =
{
  if(k==0,return(0)); \
  2*TAtwSC(k-1)
  + 2^(k-3)*BQ(k-1)
  + 2
  ;
}
check_exact(vector(100,k,k+=3; TAtwSC_by_recurrence(k)), \
            vector(100,k,k+=3; TAtwSC(k)), \
            "TAtwSC_by_recurrence 2");

{
  my(k=4);
  check_exact(2*TAtwSC(3) + 2^(4-3)*BQ(4-1) + 2,
              TAtwSC(4),
              "TAtwSC(4)");
}
{
  my(k=5);
  check_exact(2*2*TAtwSC(3)
              + 2*2^(4-3) * BQ(4-1)
              + 2^(5-3) * BQ(5-1)
              + 2*2
              + 2
              ,
              TAtwSC(5),
              "TAtwSC(5)");
}
TAtwSC_by_sum(k) =
{
  if(k<3,error("TAtwSC_by_sum(k) is for k>=3"));
  2^(k-3)*TAtwSC(3)
  + 2^(k-3)*sum(j=4,k, BQ(j-1) )
  + (2^(k-3) - 1)*2
  ;
}
check_exact(vector(100,k,k+=2; TAtwSC_by_sum(k) - TAtwSC(k)), \
            vector(100,k,k+=2; 0));   /* k>=3 */

TAtwSC_by_sum(k) =
{
  if(k<3,error("TAtwSC_by_sum(k) is for k>=3"));
  24 * 2^(k-3) - 2
  + 2^(k-3)*sum(j=3,k-1, BQ(j) )
  ;
}
check_exact(vector(100,k,k+=2; TAtwSC_by_sum(k) - TAtwSC(k)), \
            vector(100,k,k+=2; 0));   /* k>=3 */

TAtwSC_by_sum(k) =
{
  if(k<3,error("TAtwSC_by_sum(k) is for k>=3"));
  + 7/4*2^k - 2
  + 1/8 * 2^k * sum(j=0,k-1, BQ(j))
  ;
}
check_exact(vector(100,k,k+=2; TAtwSC_by_sum(k) - TAtwSC(k)), \
            vector(100,k,k+=2; 0));   /* k>=3 */

SumBQ(k) = sum(j=0,k-1,BQ(j));
gSumBQ(x) = 1/(1-x)*gBQ(x);
check_exact(gf_terms(gSumBQ(x),100), \
            vector(100,k, SumBQ(k)));

gTAtwSC(x) =
{
  -2 - 15/2*x
  -2 * 1/(1 - x)
  + (23/8)/(1 - 2*x)
  + (9/8 + 3/2*x + 5*x^2)/(1 - 2*x - 16*x^3)
  ;
}
check_exact(gf_terms(gTAtwSC(x),10), \
            vector(10,k, if(k<3,0, TAtwSC_by_sum(k))));

\\-----------------------------------------------------------------------------

TAtwC_by_LQ(k) =
{
  if(k<4,error("TAtwC_by_sum(k) is for k>=4"));
  TAtwSC(k-1)
  - (LQ(k-1)-1)
  + (TDegCount1(k-1)-1)*LQ(k-1)
  ;
}
check_exact(vector(100,k,k+=3; TAtwC_by_LQ(k)), \
            vector(100,k,k+=3; TAtwC(k)));   /* k>=4 */

TAtwC_by_LQ(k) =
{
  if(k<4,error("TAtwC_by_sum(k) is for k>=4"));
  TAtwSC(k-1) + 2^(k-3)*LQ(k-1) + 1
  ;
}
check_exact(vector(100,k,k+=3; TAtwC_by_LQ(k)), \
            vector(100,k,k+=3; TAtwC(k)));   /* k>=4 */

gTAtwC(x) =
{
  -1 - 7/2*x - 1/2*x^2
  + (-1)/(1 - x)
  + (23/16)/(1 - 2*x)
  + (9/16 + 1/2*x + 5/2*x^2)/(1 - 2*x - 16*x^3)
  ;
}
check_exact(gf_terms(gTAtwC(x),100), \
            vector(100,k, if(k<3,0, TAtwC(k))));



\\-----------------------------------------------------------------------------

TATW(k) =
{
  if(k<=3, return(gen_twindragon_area_tree_terminal_wiener_TATW_samples[k+1]));
  2*( TATW(k-1) - TAtwC(k-1) )
  + 2 * ( TDegCount1(k-1) - 1) * TAtwC(k-1)
  + ( TDegCount1(k-1) - 1)^2
  ;
}
{
  check_exact(vector(#gen_twindragon_area_tree_terminal_wiener_TATW_samples,k,k--;
                     TATW(k)),
              gen_twindragon_area_tree_terminal_wiener_TATW_samples);   /* k>=3 */
}


my(k=4); check_exact(2^(k-3)+1, 3);
my(k=5); check_exact(2^(k-3)+1, 5);
TATW(k) =
{
  if(k<=3, return(gen_twindragon_area_tree_terminal_wiener_TATW_samples[k+1]));
  2*( TATW(k-1) - TAtwC(k-1) )
  + 2 * (2^(k-3)+1) * TAtwC(k-1)
  + ( 2^(k-3)+1 )^2
  ;
}
{
  check_exact(vector(#gen_twindragon_area_tree_terminal_wiener_TATW_samples,k,k--;
                     TATW(k)),
              gen_twindragon_area_tree_terminal_wiener_TATW_samples);   /* k>=3 */
}

TATW_by_recurrence(k) =
{
  if(k<=3, return(gen_twindragon_area_tree_terminal_wiener_TATW_samples[k+1]));
  2*TATW(k-1)
  + 2 * 2^(k-3) * TAtwC(k-1)
  + 4^(k-3)
  + 2^(k-2)
  + 1
  ;
}
check_exact(vector(100,k,k--; TATW_by_recurrence(k) - TATW(k)), \
            vector(100,k,k--; 0));

{
  my(k=4);
  check_exact(2*TATW(3)
              + 2*2^(k-3)*TAtwC(k-1)
              + 4^(k-3)
              + 2^(k-2)
              + 1,
              TATW(4),
              "TATW(4)");
}
{
  my(k=5);
  check_exact(2*TATW(4)
              + 2*2^(k-3)*TAtwC(k-1)
              + 4^(k-3)
              + 2^(k-2)
              + 1
              ,
              TATW(5),
              "TATW(5)");
}
{
  my(k=5);
  check_exact(2*(2*TATW(3)
                 + 2*2^(k-1-3)*TAtwC(k-1-1)
                 + 4^(k-1-3)
                 + 2^(k-1-2)
                 + 1)
              + 2*2^(k-3)*TAtwC(k-1)
              + 4^(k-3)
              + 2^(k-2)
              + 1
              ,
              TATW(5),
              "TATW(5)");
}
{
  my(k=5);
  check_exact(2*2*TATW(3)
                 + 2*2^(k-3)*TAtwC(k-1-1)
                 + 2*4^(k-1-3)
                 + 2^(k-2)
                 + 2*1
              + 2*2^(k-3)*TAtwC(k-1)
              + 4^(k-3)
              + 2^(k-2)
              + 1
              ,
              TATW(5),
              "TATW(5)");
}
TATW_by_sum(k) =
{
  if(k<=3, return(gen_twindragon_area_tree_terminal_wiener_TATW_samples[k+1]));
  2^(k-3)*TATW(3)
  + 2^(k-2) * sum(j=3,k-1, TAtwC(j))
  + 2^(k-3)*(2^(k-2) - 1 + 2*(k-3))
  - 1
  ;
}
check_exact(vector(100,k,k--; TATW(k) - TATW_by_sum(k)), \
            vector(100,k,k--; 0));

TATW_by_sum(k) =
{
  if(k<=3, return(gen_twindragon_area_tree_terminal_wiener_TATW_samples[k+1]));
  + 2^(k-2) * sum(j=3,k-1, TAtwC(j))
  + 2^(k-3)*(2^(k-2) + 2*k + 15)
  - 1
  ;
}
check_exact(vector(100,k,k--; TATW(k) - TATW_by_sum(k)), \
            vector(100,k,k--; 0));

TATW_by_sum(k) =
{
  if(k<=3, return(gen_twindragon_area_tree_terminal_wiener_TATW_samples[k+1]));
  + 2^(k-2) * sum(j=3,k-1, TAtwC(j))
  + 2*4^(k-3)
  + (2*k + 15)*2^(k-3)
  - 1
  ;
}
check_exact(vector(100,k,k--; TATW(k) - TATW_by_sum(k)), \
            vector(100,k,k--; 0));

print("TATW k>=0  ", vector(10,k,k--; TATW(k)));

gTATW_ref(x) =
{
  + 5/16 + 1/8*x - 2*x^2 + x^3
  + (-1)/(1 - x)
  + (63/136)/(1 - 2*x)
  + (27/128)/(1 - 4*x)
  + (29/2176 + 7/136/4*(4*x) + 6/17/16*(4*x)^2)/(1 - (4*x) - 2*(4*x)^3)
  ;
}
check_exact(gf_terms(gTATW_ref(x),100), \
            vector(100,k,k--; TATW(k)));


pol_ascending_print(pol_partial_fractions(gTATW_ref(x)))

print("gLQ(4*x)");
pol_ascending_print(pol_partial_fractions(gLQ(4*x)))
check_exact(gf_terms(gLQ(4*x),10), \
            vector(10,k,k--; 4^k*LQ(k)), \
            "gLQ(4*x)");

pol_ascending_print(pol_partial_fractions(1/x * gLQ(4*x)))
check_exact(gf_terms(1/x * (gLQ(4*x)-1),10), \
            vector(10,k,k--; 4*4^k*LQ(k+1)), \
            "1/x * gLQ(4*x)");

pol_ascending_print(pol_partial_fractions(1/x^2 * gLQ(4*x)))
check_exact(gf_terms(1/x^2 * (gLQ(4*x)-1-4),10), \
            vector(10,k,k--; 16 * 4^k*LQ(k+2)), \
            "1/x^2 * gLQ(4*x)");

\\ (1 + 16*x^2)/(1 - 4*x - 128*x^3)
\\ (4 + 16*x + 128*x^2)/(1 - 4*x - 128*x^3)
\\ (32 + 128*x + 512*x^2)/(1 - 4*x - 128*x^3)
matsolve(mattranspose([1,0,16; 4,16,128; 32,128,512]), [29/2176; 7/136; 6/17]  )

gTATW(x) =
{
  5/16 + 1/8*x - 2*x^2 + x^3
  + (-1)/(1 - x)
  + (63/136)/(1 - 2*x)
  + (27/128)/(1 - 4*x)

  + 1/2176*  gLQ(4*x)
  + 19/8704 * 1/x*(gLQ(4*x)-1)
  + 9/69632 * 1/x^2*(gLQ(4*x) - 1 - 4*x)
  ;
}
pol_ascending_print(pol_partial_fractions(gTATW(x)))
check_exact(gTATW(x), gTATW_ref(x))

TATW(k) =
{
  if(k<=3, return(gen_twindragon_area_tree_terminal_wiener_TATW_samples[k+1]));
  - 1
  + (63/136)*2^k
  + (27/128)*4^k
  + (1/2176*LQ(k) + 19/8704*4*LQ(k+1) + 9/69632*16*LQ(k+2)) * 4^k
  ;
}
check_exact(gf_terms(gTATW(x),100), \
            vector(100,k,k--; TATW(k)));

TATW_formula(k) =
{
  + (1/2176*LQ(k) + 19/2176*LQ(k+1) + 9/4352*LQ(k+2) + 27/128) * 4^k
  + 63/136 * 2^k
  - 1
  ;
}
TATW(k) =
{
  if(k<=3, return(gen_twindragon_area_tree_terminal_wiener_TATW_samples[k+1]));
  TATW_formula(k);
}
check_exact(gf_terms(gTATW(x),100), \
            vector(100,k,k--; TATW(k)));

check_exact(TATW(3) != TATW_formula(3), 1);

\\-----------------------------------------------------------------------------

print("cf 1/16*TAW");
pol_ascending_print(pol_partial_fractions(gTATW(x) - 1/16*gTAW(x) ))

gTATW_by_TAW(x) =
{
  1/16*gTAW(x)
  + 7/32 * 1/(1 - 4*x)
  + 15/32 * 1/(1 - 2*x)
  - 1/(1 - x)
  + 5/16 + 1/8*x - 2*x^2 + x^3
  ;
}
\\ print("diff");
\\ pol_ascending_print(pol_partial_fractions(gTATW(x) - gTATW_by_TAW(x) ))
check_exact(gTATW_by_TAW(x), gTATW(x))


TATW_formula(k) =
{
  1/16*TAW(k)
  + 7/32 * 4^k
  + 15/32 * 2^k
  - 1
  ;
}
check_exact(gf_terms(gTATW(x),10), \
            vector(10,k,k--; TATW(k)));
