\\ Copyright 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");

b = 1+I;

LQ(k) = if(k<0,error("LQ() is for k>=0")); \
        if(k==0,return(1)); \
        if(k<=3,return(2^(k-1))); \
        LQ(k-1) + 2*LQ(k-3);
LQ = memoize(LQ);

RQ(k) = 1 + sum(j=0,k-1, LQ(j));
RQ = memoize(RQ);

\\-----------------------------------------------------------------------------

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_equal(got,want) =
{
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok"));
}

check_exact(got,want) =
{
  if(got!= want,
     print("wrong\n"); quit(1),
     print("ok"));
}
\\-----------------------------------------------------------------------------

GRQN_samples = {[
  1 + I*-1,
  4 + I*0,
  5 + I*3,
  1 + I*11,
  -23 + I*19,
  -71 + I*-5,
  -111 + I*-125,
  33 + I*-413,
  753 + I*-653,
  2353 + I*147
] / 2;}

LQ_centroid(k) = if(k<=1,return(1/2+1/2*I)); b^k/2;
LQ_centroid(2)

GRQN(k) = 1/2-1/2*I + sum(i=0,k-1, LQ(i)*(b^(i+1) + -I*LQ_centroid(i)));
check_equal(GRQN(0), 1/2-1/2*I)
check_equal(GRQN(1), 1/2-1/2*I + 3/2+1/2*I)
check_equal(GRQN(2), 1/2-1/2*I + 3/2+1/2*I + 1/2+3/2*I)
check_equal(GRQN(3), 1/2-1/2*I + 3/2+1/2*I + 1/2+3/2*I  + -3/2+3/2*I + -1/2+5/2*I)

print()
GRQN_samples[1..6]
vector(6,k,my(k=k-1); GRQN(k))
print()
check_equal(vector(length(GRQN_samples),k,my(k=k-1); GRQN(k)), GRQN_samples)

GRQN(k) = 1/2-1/2*I + sum(i=0,k-1, LQ(i)*(b*b^i - I*LQ_centroid(i)));
check_equal(vector(length(GRQN_samples),k,my(k=k-1); GRQN(k)), GRQN_samples)

GRQN(k) = 1/2-1/2*I + sum(i=0,min(k-1,1), LQ(i)*(b*b^i - I*LQ_centroid(i))) \
  + sum(i=2,k-1, LQ(i)*(b*b^i - I*LQ_centroid(i)));
check_equal(vector(length(GRQN_samples),k,my(k=k-1); GRQN(k)), GRQN_samples)

GRQN(k) = 1/2-1/2*I + sum(i=0,min(k-1,1), LQ(i)*(b*b^i - I*LQ_centroid(i))) \
  + sum(i=2,k-1, LQ(i)*(b-I/2)*b^i);
check_equal(vector(length(GRQN_samples),k,my(k=k-1); GRQN(k)), GRQN_samples)

GRQN(k) = if(k<=2,return(GRQN_samples[k+1])); \
  1/2-1/2*I + sum(i=0,min(k-1,1), (b*b^i - I*(1/2+1/2*I))) \
  + (1 + 1/2*I)*sum(i=2,k-1, LQ(i)*b^i);
check_equal(vector(length(GRQN_samples),k,my(k=k-1); GRQN(k)), GRQN_samples)

GRQN(k) = if(k<2,return(GRQN_samples[k+1])); \
  1 - 1/2*I  +  (1 + 1/2*I)*( sum(i=0,k-1, LQ(i)*b^i));
check_equal(vector(length(GRQN_samples),k,my(k=k-1); GRQN(k)), GRQN_samples)


gf_terms(g,n) = my(var=variable(g)); \
  concat(vector(min(n,valuation(g,var)), i, 0), \
         Vec(g + O(var^n)));
gLQ(x) = (1 + x^2) / (1 - x - 2*x^3);

gGRQN(x) = -1/2 \
  + (1 - 1/2*I)/(1-x) \
  + x*(1 + 1/2*I)*1/(1-x)*gLQ(b*x);
gf_terms(gGRQN(x), 5)

check_equal(gf_terms(gGRQN(x), length(GRQN_samples)), GRQN_samples)
check_equal(gf_terms(gGRQN(x),100), vector(100,k,my(k=k-1); GRQN(k)))

gGRQN(x) = -1/2 \
  + (57/82 - 21/82*I) /(1 - x) \
  + 1/82 * \
    ((25 - 20*I) + (39 - 23*I)*b*x + (8 - 31*I)*b^2*x^2) \
    / (1 - b*x - 2*b^3*x^3);
check_equal(gf_terms(gGRQN(x),100), vector(100,k,my(k=k-1); GRQN(k)))

gRQ(x) = 1/(1-x) + x/(1-x)*gLQ(x);
check_equal(gf_terms(gRQ(x),100), vector(100,k,my(k=k-1); RQ(k)))

gRQ(x) = (1 + x + x^2)/(1 - x -2*x^3);
check_equal(gf_terms(gRQ(x),100), vector(100,k,my(k=k-1); RQ(k)))


g(x) = (57/82 - 21/82*I) /(1 - x) \
  + 1/82 * ((25 - 20*I) + (39 - 23*I)*b*x + (8 - 31*I)*b^2*x^2) \
    / (1 - b*x - 2*b^3*x^3);

gg(k) = gf_terms(g(x),k+1)[k+1];

\\ Z + A*b^k*RQ(k) + B*b^(k+1)*RQ(k+1) + C*b^(k+2)*RQ(k+2) = gg(k)
{
my(k=3);
v = matsolve([ 1,     b^k*RQ(k),   b^(k+1)*RQ(k+1), b^(k+2)*RQ(k+2);
               1, b^(k+1)*RQ(k+1), b^(k+2)*RQ(k+2), b^(k+3)*RQ(k+3);
               1, b^(k+2)*RQ(k+2), b^(k+3)*RQ(k+3), b^(k+4)*RQ(k+4);
               1, b^(k+3)*RQ(k+3), b^(k+4)*RQ(k+4), b^(k+5)*RQ(k+5) ],
         [ gg(k); gg(k+1); gg(k+2); gg(k+3) ]);
}
v

gg_pow(k) = v[1,1] + v[2,1]*b^k*RQ(k) + v[3,1]*b^(k+1)*RQ(k+1) + v[4,1]*b^(k+2)*RQ(k+2);
GRQN(3)
gg_pow(3)
gg(3)
gg(8)
gg_pow(8)
GRQN(8)

GRQN_by_pow(k) = if(k<3,return(GRQN_samples[k+1]));     \
  v[1,1]                                                \
  + v[2,1]*b^k*RQ(k)                                    \
  + v[3,1]*b^(k+1)*RQ(k+1)                              \
  + v[4,1]*b^(k+2)*RQ(k+2);
check_equal(vector(100,k,my(k=k-1); GRQN_by_pow(k)), vector(100,k,my(k=k-1); GRQN(k)))


GRQN_by_pow(k) = if(k<3,return(GRQN_samples[k+1]));     \
  57/82 - 21/82*I                                       \
  + (35/41 - 15/82*I)   *b^k    *RQ(k)                                    \
  + (17/164 + 11/164*I) *b^(k+1)*RQ(k+1)                              \
  + (-11/164 + 17/164*I)*b^(k+2)*RQ(k+2);
check_equal(vector(100,k,my(k=k-1); GRQN_by_pow(k)), vector(100,k,my(k=k-1); GRQN(k)))


GRQN_by_pow(k) = if(k==0,return(1/2-1/2*I));     \
  57/82 - 21/82*I                                       \
  + (35/41 - 15/82*I) *b^k*RQ(k)                                    \
  + (3/82 + 7/41*I)   *b^k*RQ(k+1)                              \
  + (-17/82 - 11/82*I)*b^k*RQ(k+2);
check_equal(vector(100,k,my(k=k-1); GRQN_by_pow(k)), vector(100,k,my(k=k-1); GRQN(k)))



GRQN_by_pow(k) = if(k==0,return(GRQN_samples[k+1]));     \
  1/82 * ( 57 - 21*I                                    \
          + b^k* (  (70 - 15*I)  * RQ(k)                \
                  + (3 + 14*I)   * RQ(k+1)              \
                  + (-17 - 11*I) * RQ(k+2)));
check_equal(vector(100,k,my(k=k-1); GRQN_by_pow(k)), vector(100,k,my(k=k-1); GRQN(k)))

GRQN_by_pow(k) = if(k==0,return(GRQN_samples[k+1]));     \
  1/82 * ( 57 - 21*I                                    \
          + b^k* (  (70 - 15*I)  * RQ(k)                \
                  + (3 + 14*I)   * (RQ(k) + LQ(k))      \
                  + (-17 - 11*I) * (RQ(k) + LQ(k) + LQ(k+1))));
check_equal(vector(100,k,my(k=k-1); GRQN_by_pow(k)), vector(100,k,my(k=k-1); GRQN(k)))

GRQN_by_pow(k) = if(k==0,return(GRQN_samples[k+1]));     \
  1/82 * ( 57 - 21*I )                                   \
  + 1/82 * b^k* (  (70 - 15*I)  * RQ(k)                \
                 + (3 + 14*I)   * (RQ(k) + LQ(k))      \
                 + (-17 - 11*I) * (RQ(k) + LQ(k) + LQ(k+1)));
check_equal(vector(100,k,my(k=k-1); GRQN_by_pow(k)), vector(100,k,my(k=k-1); GRQN(k)))



RQ(200)/RQ(200)*1.0
RQ(201)/RQ(200)*1.0
RQ(202)/RQ(200)*1.0

cbrt(x) = x^(1/3);
C = cbrt(28 + sqrt(29*27));
r = 1/3*(C + 1/C + 1);
(70 - 15*I) \
+ (3 + 14*I) * r \
+ (-17 - 11*I) * r^2

(35/41 - 15/82*I) *1    \
+ (3/82 + 7/41*I)   *r  \
+ (-17/82 - 11/82*I)*r^2

print("r = "r)
k=100; GRQN_by_pow(k) / RQ(k) / b^k * 1.0

k=100; (35/41 - 15/82*I)  \
  + (3/82 + 7/41*I)   *r  \
  + (-17/82 - 11/82*I)*r^2

print();

m = [0,1,0;0,0,1;2,0,1];
charpoly(m)
polroots(charpoly(m))
mateigen(m,1)

f = (70-15*I)*matid(3) + ( 3 + 14*I)*m - (17 + 11*I)*m^2;
charpoly(f)
\\ x^3 + (-196 + 42*I)*x^2 + (10865 - 5330*I)*x + (-158014 + 181548*I)

f = (70)*matid(3) + ( 3)*m - (17)*m^2;
charpoly(f)
\\ real  x^3 - 196*x^2 + 11890*x - 194996

f = (-15)*matid(3) + (  14)*m - ( 11)*m^2;
charpoly(f)
\\ imag  x^3 + 42*x^2 + 1025*x + 13448
