\\ Copyright 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;
cbrt(x) = x^(1/3);

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_equal(got,want) =
{
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok"));
}

check_exact(got,want) =
{
  if(got!= want,
     print("wrong\n"); quit(1),
     print("ok"));
}

\\-----------------------------------------------------------------------------

sqrt3i = quadgen(-3 *4);
D = 1/C;
r = 1/3*(1 + C + D)
r2 = 1/6*(2 - C - D) + sqrt3i/6*(C - D)
r3 = 1/6*(2 - C - D) - sqrt3i/6*(C - D)

h = (1/6*(2 - C - D))^2 + 3*(1/6*(C - D))^2;

check_exact(h, (1/6*(2 - C - D))^2 + 3*(1/6*(C - D))^2);
check_exact(h, 1/9*C^2 + -1/9*D*C - 1/9*C + 1/9*D^2 - 1/9*D + 1/9);
check_exact(h, 1/9*C^2 - 1/9*C - 1/9*D + 1/9*D^2);
check_exact(h, 1/9*(C^2 - C - D + D^2));
check_exact(h, 1/9*((C + 1/C)^2  - 2 - C - 1/C));

\\---------------------

C = cbrt(28 + sqrt(29*27));
D = 1/C;
r2 = 1/6*(2 - C - D) + I*sqrt(3)/6*(C - D);
norm(r2)
sqrt(norm(r2))
h = (1/6*(2 - C - D))^2 + 3*(1/6*(C - D))^2

quit
