\\ Copyright 2014, 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;
cbrt(x) = x^(1/3);

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\n%s\n%s",x,y)));
  return(0);
}

check_equal(got,want) =
{
  my(d = differ(got,want));
  if(d != 0,
     print("wrong\n",d); quit(1),
     print("ok"));
}

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------

m = [1, 1, 1; r, r2, r3; r^2, r2^2, r3^2]^-1 * [0;0;1];
X = m[1,1]
Y = m[2,1]
Z = m[3,1]

check_exact(X,  1/((r-r2)*(r-r3)))
check_exact(Y, -1/((r-r2)*(r2-r3)))
check_exact(Z, -1/((r-r3)*(r3-r2)))

r = 1/3*(1 + C + D)
r2 = 1/6*(2 - C - D) + I*t/6*(C - D)
r3 = 1/6*(2 - C - D) - I*t/6*(C - D)

check_exact(r-r2, 1/2*C + 1/2*D - I*t/6*(C - D))
check_exact(r-r3, 1/2*C + 1/2*D + I*t/6*(C - D))
check_exact(r2-r3, I*t/3*(C - D))

check_exact(1/(r-r2), \
            (1/2*C + 1/2*D + I*t/6*(C - D)) \
            / ((1/2*C + 1/2*D)^2 + (t/6*(C - D))^2))

check_exact((1/2*C + 1/2*D)^2 + (t/6*(C - D))^2, \
            1/4*C^2 + 1/2*D*C + 1/4*D^2 + t^2/36*(C^2 - 2*D*C + D^2))

check_exact(1/4*C^2 + 1/2*D*C + 1/4*D^2 + 3/36*(C^2 - 2*D*C + D^2), \
            1/3*C^2 + 1/3*D*C + 1/3*D^2)

\\-----------------------------------------------------------------------------
t = sqrt(3);
C = cbrt(28 + sqrt(29*27));
D = 1/C;
r = 1/3*(1 + C + D)
r2 = 1/6*(2 - C - D) + I*t/6*(C - D)
r3 = 1/6*(2 - C - D) - I*t/6*(C - D)

m = [1, 1, 1; r, r2, r3; r^2, r2^2, r3^2]^-1 * [0;0;1];
X = m[1,1]
Y = m[2,1]
Z = m[3,1]

check_equal(X,  1/((r-r2)*(r-r3)))
check_equal(X, 1/((1/2*C + 1/2*D)^2 + (t/6*(C - D))^2))
check_equal(X, 1/(1/4*(C + D)^2 + 1/12*(C - D)^2))
check_equal(X, 1/(1/4*(C + D)^2 + 1/12*(C - D)^2))
check_equal(X, 3/(C^2 + D*C + D^2))
check_equal(X, 3/(C^2 + 1 + 1/C^2))

check_equal(r-r2, 1/2*C + 1/2*D - I*t/6*(C - D))
check_equal(r2-r3, I*t/3*(C - D))
check_equal(Y, -1/((r-r2)*(r2-r3)))
check_equal(Y, -1/((1/2*C + 1/2*D - I*t/6*(C - D))*(I*t/3*(C - D))))
check_equal(Y, -1/((1/2*C + 1/2*D)*(I*t/3*(C - D)) - I*t/6*(C - D)*(I*t/3*(C - D))))
check_equal(Y, -1/(t/6*(C+D)*I*(C-D) + 1/6*(C-D)*(C-D)))
check_equal(Y, -6/((C-D)*(C-D) + I*t*(C+D)*(C-D)))

check_equal(Y, -6*((C-D)*(C-D) - I*t*(C+D)*(C-D)) \
               /( (C^3-D^3)*(C-D)*4 ))
check_equal(Y, -3/2 * ((C-D)*(C-D) - I*t*(C+D)*(C-D)) \
                      /( (C^3-D^3)*(C-D) ))

check_equal(Y, -3/2 * ((C-1/C)*(C-1/C) - I*t*(C+1/C)*(C-1/C)) \
                      /( (C^3-1/C^3)*(C-1/C) ))
check_equal(Y, -3/2 * C^2 * ((C^2-1)*(C^2-1) - I*t*(C^4-1)) \
                      /( (C^6-1)*(C^2-1) ))

\\-----------------------------------------------------------------------------
C = 'C;
X = 3/(C^2 + 1 + 1/C^2);

sqrt3i = quadgen(-12);  /* sqrt(3)*I */
cubic_r2_from_C = 1/6*(2-'C-1/'C) + sqrt3i/6*('C-1/'C);
r_from_C = 1/3*(1 + 'C + 1/'C);

\\
\\--------------------------------------
\\ characteristic equation x^3 - x^2 - 2 = 0
\\ real root D^(1/3) + (1/9)*D^(-1/3) + 1/3 = 1.6956207695598620
\\ where D=28/27 + (1/9)*sqrt(29*3) = 28/27 + sqrt(29/27)
\\
\\ polroots(x^3 - x^2 - 2)
\\ polroots(x^4 - 2*x^3 + x^2 - 2*x + 2)
\\ x^3 + px + q = 0
\\ x = cbrt(q/2 + sqrt(p^3/27 + q^2/4)) + cbrt(q/2 - sqrt(p^3/27 + q^2/4))
\\
\\ change of variable x=y+1/3; x^3 - x^2 - 2
\\ y^3 - 1/3*y - 56/27 = 0
\\ p = -1/3; q=-56/27
\\ y = (-28/27 + sqrt(29/27))^(1/3) + (-28/27 - sqrt(29/27))^(1/3)
\\ x = 1/3 + ...
\\
\\ other root -0.3478103847799310287081835501 - 1.028852254136692403433083008*I
\\ abs() = 1.086052035863345163204928125


\\ abs(r2)
\\
check_exact(norm(cubic_r2_from_C), \
            (1/6*(2 - 'C - 1/'C))^2 + 3*(1/6*('C - 1/'C))^2  );
check_exact(norm(cubic_r2_from_C), \
            ('C^4 - 'C^3 - 'C + 1)/(9*'C^2)  );
check_exact(norm(cubic_r2_from_C), \
            1/9*('C^2 - 'C - 1/'C + 1/'C^2)  );
check_exact(norm(cubic_r2_from_C), \
            1/9*('C^2 + 1/'C^2 - 'C - 1/'C)  );
check_exact(norm(cubic_r2_from_C), \
            1/9*(('C + 1/'C)^2 - 2 - 'C - 1/'C)  );
check_exact(norm(cubic_r2_from_C), \
            1/9*(('C + 1/'C)^2 - ('C + 1/'C) - 2)  );

check_exact((-1)^2 - 4*1*-2,  9);    /* discriminant */
\\ sqrt(3)/6 *(3-1/4)

check_exact(norm(cubic_r2_from_C), \
            1/9*((1 + 'C + 1/'C)^2 - (2*'C + 1 + 2/'C) - ('C + 1/'C) - 2)  );
check_exact(norm(cubic_r2_from_C), \
            1/9*((1 + 'C + 1/'C)^2 - 3*(1 + 'C + 1/'C))  );

check_exact(norm(cubic_r2_from_C), \
            r_from_C^2 - r_from_C  );

\\ r^3 - r^2 - 2 == 0
\\ r^2 - r == 2/r

check_equal(norm(r2), r^2 - r  );
check_equal(norm(r2), 2/r );

norm(r2) - 2*(D + 1/D)

quit
