\\ Copyright 2014, 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


read("../tools/memoize.gp");
read("pol-pfrac.gp");

default(recover,0);
check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

vector_modulo(v,i) = v[(i% #v)+1];
BlobDeltaLen(k) = if(k<4,0, 1/3*(2^ceil(k/2) - vector_modulo(BlobDeltaLen_vector,k)));
BlobDeltaLen_vector = [4, 5, 5, 7 ];
TotalNonBlob(k) = if(k<#TotalNonBlob_initial, TotalNonBlob_initial[k+1], 6*k-13);
TotalNonBlob_initial = [1, 2, 4, 8, 12];

BlobList(k) = {
  if(k<0,error("BlobList() is for k>=0")); \
  my(i=0,ret=List([]));
  while(i<=k, listput(ret,i);i++);
  i -= 3;  /* from k+1 down to k-2 */
  while(i>=0, listput(ret,i);i--);
  Vec(ret);
}

gf_terms(g,n) = my(x=variable(g)); \
  concat(vector(min(n,valuation(g,x)), i, 0), \
         Vec(g + O(x^n)));
gf_term(g,n) = my(v=gf_terms(g,n+1)); v[#v];

\\-----------------------------------------------------------------------------


BlobPathLen_by_BlobDeltaLen(k) = if(k<4,0, BlobDeltaLen(k)*vector_modulo([2,1],k));

BlobPathLen(k) = if(k<4,0, \
  1/3*(2*2^floor(k/2) - vector_modulo([8,5,10,7],k)));
check_exact(vector(64,k,my(k=k-1); BlobPathLen(k)), \
            vector(64,k,my(k=k-1); BlobPathLen_by_BlobDeltaLen(k)));
\\ even terms 2*A084639

BlobPathLen(k) = if(k<4,0, \
  my(h = floor(k/2)); \
  1/3*(2*2^h + vector_modulo([-8,-10],h)) + (k%2));
check_exact(vector(64,k,my(k=k-1); BlobPathLen(k)), \
            vector(64,k,my(k=k-1); BlobPathLen_by_BlobDeltaLen(k)));

BlobPathLen(k) = if(k<4,0, \
  my(h = floor(k/2)); \
  2/3*(2^h - vector_modulo([4,5],h)) + (k%2));
check_exact(vector(64,k,my(k=k-1); BlobPathLen(k)), \
            vector(64,k,my(k=k-1); BlobPathLen_by_BlobDeltaLen(k)));

print("BlobDeltaLen ", vector(16,k,my(k=k-1); BlobDeltaLen(k)));
print("BlobPathLen ",  vector(16,k,my(k=k-1); BlobPathLen(k)));

\\-----------------------------------------------------------------------------

BlobPathLenCumulative_by_sum(k) = sum(j=1,k, BlobPathLen(j));

BlobPathLenCumulative(k) = \
  sum(j=4,k, 1/3*(2*2^floor(j/2) - vector_modulo([8,5,10,7],j)));
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));
vector(16,k,my(k=k+7); BlobPathLenCumulative(2*k+1))

BlobPathLenCumulative(k) = \
  1/3 * ( 2*sum(j=4,k, 2^floor(j/2)) \
          - sum(j=4,k, vector_modulo([8,5,10,7],j)) );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  2/3 * sum(j=4,k, 2^floor(j/2)) \
  - 1/3 * sum(j=4,k, vector_modulo([8,5,10,7],j));
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  -4 \
  + 2/3 * ( sum(j=0,k, 2^floor(j/2)) ) \
  - 1/3 * sum(j=4,k, vector_modulo([8,5,10,7],j));
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  -4 \
  + 2/3 * ( 2^(floor(k/2)+1)-1  + 2^(floor(k/2)+(k%2))-1 ) \
  - 1/3 * sum(j=4,k, vector_modulo([8,5,10,7],j));
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  -4 - 4/3 \
  + 2/3 * ( 2*2^floor(k/2)  + 2^floor(k/2)*vector_modulo([1,2],k) ) \
  - 1/3 * sum(j=4,k, vector_modulo([8,5,10,7],j));
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  -16/3 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * (-30 + sum(j=0,k, vector_modulo([8,5,10,7],j)) );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * ( sum(j=0,k, vector_modulo([8,5,10,7],j)) );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * ( 8*(floor(k/4)+1) \
          + 5*(floor((k-1)/4)+1) \
          + 10*(floor((k-2)/4)+1) \
          + 7*(floor((k-3)/4)+1) );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * ( 8*floor(k/4) + 8 \
          + 5*floor((k-1)/4) + 5 \
          + 10*floor((k-2)/4) + 10 \
          + 7*floor((k-3)/4) +7 );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 - 10 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * ( 8*floor(k/4)  \
          + 5*floor((k-1)/4)  \
          + 10*floor((k-2)/4)  \
          + 7*floor((k-3)/4)  );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 - 10 \
  + 2/3 * ( 2^floor(k/2)*vector_modulo([3,4],k) ) \
  - 1/3 * ( 30*floor(k/4)  \
            - vector_modulo([5,0,0,0],k)  \
            - vector_modulo([10,10,0,0],k)  \
            - vector_modulo([7,7,7,0],k)   );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  14/3 - 10 \
  + 1/3 * ( 2^floor(k/2)*vector_modulo([6,8],k) ) \
  + 1/3 * ( -30*floor(k/4)  \
            + vector_modulo([22,17,7,0],k)   );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  1/3 * ( 2^floor(k/2)*vector_modulo([6,8],k)   \
          - 30/4*k  \
          + vector_modulo([6,1+30/4,-9+2*30/4,-16+3*30/4],k)   );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = \
  if(k<4,return(0)); \
  1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)   \
          - 30/2*k  \
          + vector_modulo([12, 17, 12, 13],k)   );
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

BlobPathLenCumulative(k) = {
  if(k<4,0,
     1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
             - 15*k
             + vector_modulo([12, 17, 12, 13],k)   ));
}
check_exact(vector(64,k,my(k=k-1); BlobPathLenCumulative(k)), \
            vector(64,k,my(k=k-1); BlobPathLenCumulative_by_sum(k)));

vector(16,k,my(k=k+3); BlobPathLenCumulative(k))

\\-----------------------------------------------------------------------------

Diameter_by_blobs(k) = \
  BlobPathLenCumulative(k) + BlobPathLenCumulative(k-2) + TotalNonBlob(k);
vector(16,k,my(k=k+7); Diameter_by_blobs(k))

Diameter(k) = {
  if(k<4,0,
     1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
             - 15*k
             + vector_modulo([12, 17, 12, 13],k)   ))
  +
  if((k-2)<4,0,
     1/6 * ( 2^floor((k-2)/2)*vector_modulo([12,16],(k-2))
             - 15*(k-2)
             + vector_modulo([12, 17, 12, 13],(k-2))   ))
  + TotalNonBlob(k);
}
check_exact(vector(64,k,my(k=k-1); Diameter(k)), \
            vector(64,k,my(k=k-1); Diameter_by_blobs(k)));

Diameter(k) = {
  if(k<4,0,
     1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
             - 15*k
             + vector_modulo([12, 17, 12, 13],k)   ))
  +
  if(k<6,0,
     1/6 * ( 2^floor(k/2)*vector_modulo([6,8],k)
             - 15*k
             + vector_modulo([42, 43, 42, 47],k)   ))
  + 6*k-13
  + if(k<5, [1-(6*0-13), 2-(6*1-13), 4-(6*2-13), 8-(6*3-13), 12-(6*4-13)][k+1], 0);
}
check_exact(vector(64,k,my(k=k-1); Diameter(k)), \
            vector(64,k,my(k=k-1); Diameter_by_blobs(k)));

Diameter(k) = {
     1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
             - 15*k
             + vector_modulo([12, 17, 12, 13],k)   )
  +
  if(k<6,0,
     1/6 * ( 2^floor(k/2)*vector_modulo([6,8],k)
             - 15*k
             + vector_modulo([42, 43, 42, 47],k)   ))
  + 6*k
  - 13
  + if(k<5, [10, 6, 4, 3, 1][k+1], 0);
}
check_exact(vector(64,k,my(k=k-1); Diameter(k)), \
            vector(64,k,my(k=k-1); Diameter_by_blobs(k)));

Diameter(k) = {
     1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
             - 15*k
             + vector_modulo([12, 17, 12, 13],k)   )
  +
     1/6 * ( 2^floor(k/2)*vector_modulo([6,8],k)
             - 15*k
             + vector_modulo([42, 43, 42, 47],k)   )
  + 6*k
  - 13
  + if(k<5, [2, 0, 0, 0, 0][k+1], 0);
}
check_exact(vector(64,k,my(k=k-1); Diameter(k)), \
            vector(64,k,my(k=k-1); Diameter_by_blobs(k)));

Diameter(k) = {
  1/6 * ( 2^floor(k/2)*vector_modulo([12,16],k)
          + 2^floor(k/2)*vector_modulo([6,8],k)
          + vector_modulo([54, 60, 54, 60],k)   )
  + k
  - 13
  + if(k<5, [2, 0, 0, 0, 0][k+1], 0);
}
check_exact(vector(64,k,my(k=k-1); Diameter(k)), \
            vector(64,k,my(k=k-1); Diameter_by_blobs(k)));

Diameter(k) = {
  2^floor(k/2)*vector_modulo([3, 4],k)
  + vector_modulo([0,1],k)
  + k - 4
  + if(k<5, [2, 0, 0, 0, 0][k+1], 0);
}
check_exact(vector(64,k,my(k=k-1); Diameter(k)), \
            vector(64,k,my(k=k-1); Diameter_by_blobs(k)));

Diameter(k) = {
  if(k==0,   1,
     k%2==0, 3*2^floor(k/2) + k - 4,
             4*2^floor(k/2) + k - 3);
}
check_exact(vector(64,k,my(k=k-1); Diameter(k)), \
            vector(64,k,my(k=k-1); Diameter_by_blobs(k)));

print("Diameter ", vector(16,k,my(k=k-1); Diameter(k)))


\\-----------------------------------------------------------------------------

\\ blob k down to 4 and the non-blob segments between
CumulativePathLen_by_sum(k) =
{
  if(k<4, error());
  sum(j=4,k,
          BlobPathLen(j) * vector_modulo([2,1], j)
          + if(j>=5,3,0))  \\ bridge before j>=5
  + 7;
}

CumulativePathLen(k) =
{
  if(k<4, error());
  1/3* sum(j=4,k,
             (2*2^floor(j/2) - vector_modulo([8,5,10,7],j))
             * vector_modulo([2,1], j))
  + if(k>=5, 3*k-12)
  + 7
  ;
}
check_exact(vector(64,k,my(k=k+3); CumulativePathLen(k)) \
            - vector(64,k,my(k=k+3); CumulativePathLen_by_sum(k)), \
            vector(64,k,my(k=k+3); 0));

CumulativePathLen(k) =
{
  if(k<4, error());
  2/3* (sum(j=0,k, 2^floor(j/2) * vector_modulo([2,1], j)))
  - 2/3* (sum(j=0,k, vector_modulo([4,5/2,5,7/2],j) * vector_modulo([2,1], j)))
  + 3*k + 5
  ;
}
check_exact(vector(64,k,my(k=k+3); CumulativePathLen(k)) \
            - vector(64,k,my(k=k+3); CumulativePathLen_by_sum(k)), \
            vector(64,k,my(k=k+3); 0));

CumulativePathLen(k) =
{
  if(k<4, error());
  2/3* (sum(j=0,k, 2^floor(j/2) * vector_modulo([2,1], j)))
  +  (sum(j=0,k, vector_modulo([-16/3, -5/3, -20/3, -7/3],j)))
  + 3*k + 5
  ;
}
check_exact(vector(64,k,my(k=k+3); CumulativePathLen(k)) \
            - vector(64,k,my(k=k+3); CumulativePathLen_by_sum(k)), \
            vector(64,k,my(k=k+3); 0));

CumulativePathLen(k) =
{
  if(k<4, error());
  2/3* (sum(j=0,k, 2^floor(j/2) * vector_modulo([2,1], j)))
  - k
  + vector_modulo([-1/3, 2, -2/3, 1],k)
  ;
}
check_exact(vector(64,k,my(k=k+3); CumulativePathLen(k)) \
            - vector(64,k,my(k=k+3); CumulativePathLen_by_sum(k)), \
            vector(64,k,my(k=k+3); 0));

CumulativePathLen(k) =
{
  if(k<4, error());
  if(k%2==0,
           10/3* 2^floor(k/2),
           4* 2^floor(k/2))
  - k
  - vector_modulo([7/3, 0, 8/3, 1],k)
  ;
}
check_exact(vector(64,k,my(k=k+3); CumulativePathLen(k)) \
            - vector(64,k,my(k=k+3); CumulativePathLen_by_sum(k)), \
            vector(64,k,my(k=k+3); 0));


\\-----------------------------------------------------------------------------

\\ BlobDiameterBound(4) = 2
\\ BlobDiameterBound(5) = 5
\\ BlobDiameterBound(k+2) = 2*BlobDiameterBound(k) + 2
\\ 2*5+2
\\ 2*(2*5+2) + 2
\\ 5*2^(k/2) + 2*(2^(k/2)-1)


BlobDiameterBound_recurrence(k) =
{
  if(k<4, error());
  if(k-4 < #initial, initial[k-4+1],
     2^(step/2)*BlobDiameterBound_recurrence(k-step)
     + vector_modulo(add_vector,k)
     \\ + add_vector[1]
    );
}

\\ g(x) = 2 + 5*x + 8*x^2 + 12*x^3 + 4*x^4*g(x) + x^4*8/(1-x);
g_ref(x) = 1/(1-4*x^4) * (2 + 5*x + 8*x^2 + 12*x^3 + x^4*8/(1-x));
g(x) = (-4*x^4 + 4*x^3 + 3*x^2 + 3*x + 2)/(4*x^5 - 4*x^4 - x + 1);
check_exact(g(x), g_ref(x));
g(x) = -8/3*1/(1-x) + 1/2*(15*x + 10)/(1 - 2*x^2) - 1/6*(2 - x)/(1 + 2*x^2);
check_exact(g(x), g_ref(x));
g(x) = -8/3*1/(1-x) + 1/3*(14 + 23*x + 32*x^2 + 44*x^3)/(1 - 4*x^4);
check_exact(g(x), g_ref(x));

\\ step = 2;
\\ initial = [2,5,8,12,18,26];  \\ diameter 6 of
\\ add_vector = [7,7,5,4];      \\ possible 3,4 or 2,3 or 2,2
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   if(k==4, 2,
\\      k==5, 5,
\\      k==6, 8,
\\      k==7, 12,
\\      vector_modulo([71/48, 31/16, 71/48, 31/16],k) * 2^floor(k/2)
\\      - vector_modulo([17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 5, \\ 7
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\ \\
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6,
\\                       17/3, 5, 19/3, 6
\\                      ],k)
\\      - if(k==7,1);
\\      - if(k==7,1);
\\      );
\\ }
\\ check_exact([5/3, 2, 5/3, 2] - [71/48, 31/16, 71/48, 31/16], [3/16, 1/16, 3/16, 1/16]) \\ ok

\\ step = 2;
\\ initial = [2,5,8,12,18,26];  \\ diameter 6 of
\\ add_vector = [7];            \\ possible 3,4
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   if(k==4,2, k==5,5, k==6,8, k==7,12,
\\      vector_modulo([25/16, 33/16],k) * 2^floor(k/2)
\\      - 7);
\\ }
\\ check_exact([5/3, 2] - [25/16, 33/16], [5/48, -1/16]) \\ bad

\\ step = 2;
\\ initial = [2,5,8,12,18,26,36];  \\ diameter 7 of
\\ add_vector = [7];               \\ possible 3,4
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   if(k==4,2, k==5,5, k==6,8, k==7,12, k==8,18, k==9,26,
\\      vector_modulo([43/32, 33/16],k) * 2^floor(k/2)
\\      - 7);
\\ }
\\ check_exact([5/3, 2] - [43/32, 33/16], [31/96, -1/16]) \\ bad

step = 2;
initial = [2,5,8,12,18,26,36,52];  \\ diameter 8 of
add_vector = [7];                  \\ possible 3,4
BlobDiameterBound(k) =
{
  if(k<4, error());
  if(k==4,2, k==5,5, k==6,8, k==7,12, k==8,18, k==9,26, k==10,36, k==11,52,
     vector_modulo([43/32, 59/32],k) * 2^floor(k/2)
     - 7);
}
check_exact([5/3, 2] - [43/32, 59/32], [31/96, 5/32]) \\ bad

\\----

\\ initial = [2,5,8,12];   \\ diameter by 4
\\ add = 8;
\\ step = 4;
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   vector_modulo([7/6, 23/12, 20/15, 22/12],k) * 2^floor(k/2)
\\   - 8/3;
\\ }
\\ check_exact([5/3, 2, 5/3, 2] - [7/6, 23/12, 20/15, 22/12], [1/2, 1/12, 1/3, 1/6]) \\  ok

\\ initial = [2,3];  \\ start eccentricity
\\ add = 5;          \\ possible 3 at start, 2 at other
\\ step = 2;
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   vector_modulo([7/4, 8/4],k) * 2^floor(k/2)
\\   - 5;
\\ }
\\ check_exact([5/3, 2] - [7/4, 8/4], [-1/12, 0]) \\  gives too much away, eventually

\\ initial = [2,3,5,8];  \\ start eccentricity
\\ add = 5;              \\ possible 3 at start, 2 at other
\\ step = 2;
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   if(k==4,2, k==5,3,
\\      vector_modulo([10/8, 13/8],k) * 2^floor(k/2)
\\      - 5);
\\ }
\\ check_exact([5/3, 2] - [10/8, 13/8], [5/12, 3/8]) \\ ok

\\ step = 2;
\\ initial = [2,5];  \\ diameter 2 of
\\ add = 5;          \\ possible 3 at start/end, 2 at other
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   vector_modulo([21/12, 10/4],k) * 2^floor(k/2)
\\   - 5;
\\ }
\\ check_exact([5/3, 2] - [21/12, 10/4], [-1/12, -1/2]) \\  gives too much away

\\ step = 2;
\\ initial = [2,5,8,12];  \\ diameter 4 of
\\ add = 5;               \\ possible 3 at start/end, 2 at other
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   if(k==4,2, k==5,5,
\\      vector_modulo([13/8, 17/8],k) * 2^floor(k/2)
\\      - 5);
\\ }
\\ check_exact([5/3, 2] - [13/8, 17/8], [1/24, -1/8]) \\  gives too much away

\\ step = 2;
\\ initial = [2,5,8,12,18];  \\ diameter 5 of
\\ add = 5;                  \\ possible 3 at start/end, 2 at other
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   if(k==4,2, k==5,5, k==6,8,
\\      vector_modulo([23/16, 17/8],k) * 2^floor(k/2)
\\      - 5);
\\ }
\\ check_exact([5/3, 2] - [23/16, 17/8], [11/48, -1/8]) \\  gives too much away

\\ step = 2;
\\ initial = [2,5,8,12,18,26];  \\ diameter 6 of
\\ add_vector = [5];            \\ possible 3 at start/end, 2 at other
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   if(k==4,2, k==5,5, k==6,8, k==7,12,
\\      vector_modulo([23/16, 31/16],k) * 2^floor(k/2)
\\      - 5);
\\ }
\\ check_exact([5/3, 2] - [23/16, 31/16], [11/48, 1/16]) \\ ok

\\ initial = [2,3,5,8];    \\ start eccentricity
\\ add = 6;                \\ +2 start  +4 expand
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   vector_modulo([1, 5/4, 7/8, 5/4],k) * 2^floor(k/2)
\\   - 2;
\\ }
\\ check_exact(vector(64,k,my(k=k+3); BlobDiameterBound(k)) \
\\             - vector(64,k,my(k=k+3); BlobDiameterBound_recurrence(k)), \
\\             vector(64,k,my(k=k+3); 0));
\\ check_exact([5/3, 2, 5/3, 2] - [1, 5/4, 7/8, 5/4], [2/3, 3/4, 19/24, 3/4]) \\  ok

\\ for [2,5] + 3     is not quite right
\\ BlobDiameterBound(k) =
\\ {
\\   if(k<4, error());
\\   vector_modulo([5/4, 2],k) * 2^floor(k/2)
\\   - 3;
\\ }

\\ BlobDiameterBound_recurrence = memoize(BlobDiameterBound_recurrence);
\\ print("BlobDiameterBound_recurrence ", vector(16,k,my(k=k+3); BlobDiameterBound_recurrence(k)))
\\ print("g                            ", gf_terms(g(x),16));

print("BlobDiameterBound_recurrence ", vector(10,k,my(k=k+3); BlobDiameterBound_recurrence(k)))
print("BlobDiameterBound            ", vector(10,k,my(k=k+3); BlobDiameterBound_recurrence(k)))

check_exact(vector(256,k,my(k=k+3); BlobDiameterBound(k)) \
            - vector(256,k,my(k=k+3); BlobDiameterBound_recurrence(k)), \
            vector(256,k,my(k=k+3); 0));
quit

\\-----------------------------------------------------------------------------
\\ blob diameter exactly

BlobDiameterExact_vector = [2,5,8,12,18,26,36,52];
BlobDiameterExact(k) =
{
  if(k<4, error());
  my(i = k-3);
  if(i <= #BlobDiameterExact_vector, BlobDiameterExact_vector[i],
     0);
}
check_exact(vector(#BlobDiameterExact_vector,k,my(k=k+3); BlobDiameterExact(k) <= BlobDiameterBound(k)), \
            vector(#BlobDiameterExact_vector,k,my(k=k+3); 1));


\\-----------------------------------------------------------------------------

\\ blob k then k-2 down to 4 and the non-blob segments between
TailLen_by_sum(k) =
{
  if(k<4, error());
  BlobDeltaLen(k) * vector_modulo([2,1], k)
  + sum(j=4,k-2,
          BlobDeltaLen(j) * vector_modulo([2,1], j)
          + 3)  \\ bridge after
  + if(k==4, 5, 7)
  ;
}

TailLen(k) =
{
  if(k<4, error());
  1/3*(2^ceil(k/2) - vector_modulo(BlobDeltaLen_vector,k)) * vector_modulo([2,1], k)
  + sum(j=4,k-2,
          1/3*(2^ceil(j/2) - vector_modulo(BlobDeltaLen_vector,j)) * vector_modulo([2,1], j)
          + 3)  \\ bridge after
  + if(k==4, 5, 7)
  ;
}
check_exact(vector(64,k,my(k=k+3); TailLen(k)) \
            - vector(64,k,my(k=k+3); TailLen_by_sum(k)), \
            vector(64,k,my(k=k+3); 0));

TailLen(k) =
{
  if(k<4, error());
  1/3*(2^ceil(k/2) - vector_modulo(BlobDeltaLen_vector,k)) * vector_modulo([2,1], k)
  + sum(j=4,k-2,
          1/3*(2^ceil(j/2) - vector_modulo(BlobDeltaLen_vector,j)) * vector_modulo([2,1], j)
          )  \\ bridge after
  + 3*k - 15 + 7
  + if(k==4, 1)
  ;
}
check_exact(vector(64,k,my(k=k+3); TailLen(k)) \
            - vector(64,k,my(k=k+3); TailLen_by_sum(k)), \
            vector(64,k,my(k=k+3); 0));

TailLen(k) =
{
  if(k<4, error());
  vector_modulo([2/3, 2/3], k) * 2^floor(k/2)
  - vector_modulo([8/3 - 3, 5/3 - 3, 10/3 - 3, 7/3 - 3],k) - 6
  + 3*k
  + 1/6 * vector_modulo([12,16],k-2) * 2^floor((k-2)/2)
  - 1/6 *15*k
  + 1/6 *vector_modulo([12, 17, 12, 13],k-2)
  ;
}
check_exact(vector(64,k,my(k=k+3); TailLen(k)) \
            - vector(64,k,my(k=k+3); TailLen_by_sum(k)), \
            vector(64,k,my(k=k+3); 0));

TailLen(k) =
{
  if(k<4, error());
  vector_modulo([5/3, 2], k) * 2^floor(k/2)
  + 1/2*k
  - vector_modulo([11/3, 5/2, 13/3, 5/2], k)
  ;
}
check_exact(vector(64,k,my(k=k+3); TailLen(k)) \
            - vector(64,k,my(k=k+3); TailLen_by_sum(k)), \
            vector(64,k,my(k=k+3); 0));



\\-----------------------------------------------------------------------------

\\ blob k then k-2 down to 4 and the non-blob segments between
Diff_ref(k) = TailLen(k) - BlobDiameterBound(k);

\\ Diff(k) =
\\ {
\\   vector_modulo([5/3, 2, 5/3, 2], k) * 2^floor(k/2)
\\   + 1/2*k
\\   - vector_modulo([11/3, 5/2, 13/3, 5/2], k)
\\   - (
\\      vector_modulo([1, 5/4, 7/8, 5/4],k) * 2^floor(k/2)
\\      - 2
\\     )
\\   ;
\\ }
\\ check_exact(vector(64,k,my(k=k+3); Diff(k)) \
\\             - vector(64,k,my(k=k+3); Diff_ref(k)), \
\\             vector(64,k,my(k=k+3); 0));

Diff(k) =
{
  if(k==4,3, k==5,3, k==6,4, k==7,5,
     vector_modulo([11/48, 1/16], k) * 2^floor(k/2)
     + 1/2*k
     + vector_modulo([4/3, 5/2, 2/3, 5/2], k)
     );
}
check_exact(vector(64,k,my(k=k+3); Diff(k)) \
            - vector(64,k,my(k=k+3); Diff_ref(k)), \
            vector(64,k,my(k=k+3); 0));



print("Diff ", vector(16,k,my(k=k+3); Diff_ref(k)))
