#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::ComplexMinus 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyDragon 'xy_rotate_minus90';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 12;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::ComplexMinus->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $x = 0;
my $y = 0;
my $dx = 0;
my $dy = 1;
while ($path->xy_to_n($x+1,$y) <= $n_hi) {
  $x++;
}
### edge x: $x
my $dir = 1;

my $orig_x = $x;
my $orig_y = $y;
my $orig_dx = $dx;
my $orig_dy = $dy;

#
# +-----+-----+-----+
# |     |     |     |
# | NW  |  N  |  NE |
# |     |     |     |         dx,dy
# +-----+-----+-----+           ^
# |     |     |                 |
# |  W  |  *  |        lx,ly <--*--> rx,ry
# |     |     |
# +-----+-----+
# |     |     |
# | SW  |  S  |
# |     |     |
# +-----+-----+
#
$Printwrap = 2;
printwrap("\\draw ");
my ($prev_x, $prev_y);
{
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  $prev_x = 2*$x-$dx+$rx;
  $prev_y = 2*$y-$dy+$ry;
  $prev_x /= 10;
  $prev_y /= 10;
  printwrap("($prev_x,$prev_y)");
}
my $ComplexBoundary = 0;
do {
  ### at: "$x,$y  dxdy=$dx,$dy"
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  {
    my $tx = 2*$x+$dx+$rx;
    my $ty = 2*$y+$dy+$ry;
    $tx /= 10;
    $ty /= 10;
    my $delta_x = $tx - $prev_x;
    my $delta_y = $ty - $prev_y;
    $delta_x = ($delta_x ? sprintf('%.1f', $delta_x) : 0);
    $delta_y = ($delta_y ? sprintf('%.1f', $delta_y) : 0);
    printwrap("--++($delta_x,$delta_y)");
    $prev_x = $tx;
    $prev_y = $ty;
  }
  $ComplexBoundary++;

  if ($ComplexBoundary % 10000 == 0) {
    printwrap("; \\draw ($prev_x,$prev_y)");    
  }

  my $x2 = $x + $dx + $rx;
  my $y2 = $y + $dy + $ry;
  if ($path->xy_to_n($x2,$y2) <= $n_hi) {
    ### diag, turn -1 ...
    ($dx,$dy) = ($rx,$ry);
    $x = $x2;
    $y = $y2;
  } else {
    $x2 = $x + $dx;
    $y2 = $y + $dy;
    if ($path->xy_to_n($x2,$y2) <= $n_hi) {
      ### straight
      $x = $x2;
      $y = $y2;
    } else {
      ### unmoved, turn -1 ...
      ($dx,$dy) = (-$rx,-$ry);
    }
  }

} until ($x == $orig_x && $y == $orig_y
         && $dx == $orig_dx && $dy == $orig_dy);


# $Printwrap += 2;
# $segments->printwrap;
printwrap(";\n");

# /2 because 2*X,2*Y coordinates
# my $ComplexBoundary = $segments->length / 2;
print "  % boundary length $ComplexBoundary\n";
print "% GP-Test  ComplexBoundary($level) == $ComplexBoundary\n";

print "  % end generated\n";
exit 0;
