\\ Copyright 2016, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");
default(seriesprecision,7);


\\-----------------------------------------------------------------------------
\\ sum(i=1,inf, 1/(2^x-1))
\\ 1.60669515241529176378330
\\ A065442 decimal Erdos-Borwein
\\ A211706 binary
\\ Finch alpha, page 355
\\ Joerg Arndt Fast computation via Lambert series:
\\   sum(n>=1, x^(n^2) *(1+x^n)/(1-x^n) ) where x=1/2
\\ Charles R Greathouse
\\   suminf(n=1, my(k=.5^n); k^n*(1+k)/(1-k))
\\ http://www.people.fas.harvard.edu/~sfinch/constant/dig/dig.html
\\ Crandall
\\   suminf(m=1, 1/2^(m^2) * (2^m + 1) / (2^m - 1))
\\   suminf(m=1, 1/2^(m^2) * (1 + suminf(i=1, 2/2^(i*m))))
\\   add bit at m^2 then each multiple of m

\\ sum(i=1,inf, 1/(2^x-1)^2)
\\ = sum(i=1,inf,  1/(2^x-1)^2)
\\ 1.13733873634419659669691
\\ A065443 decimal
\\ (OEIS no binary)
\\ Finch beta

\\ 1/prodinf(k=1, 1-1/2^k)
\\ 0.288788095086602421278899
\\ A048651 decimal
\\ (OEIS no binary)
\\ Finch Q, page 356
\\ recip 3.46274661945506361153795
\\ A065446

\\-----------------------------------------------------------------------------

f(x) = 1/sqrt(2^x + 1);
{
  for(start=0,1,
     my(value=sum(i=start,500,f(i)));
     print(value*1.0);
     print(digits(floor(10^20 * value)));
     print(binary(floor(2^20 * value))));
}
f(x)

print("f'");
f(x)'
\\ d/dx 1/(2^x+1)
1/(2^x+1)^2 * -1 * log(2) * 2^x
f1(x) = 1/( (2^(1/2))^x + (2^(-1/2))^x)^2 * -1 * log(2);
f1(x)
f1(0)
1/( 1 + 1 )^2 * -1 * log(2)
1/4 * -1 * log(2)
quit

print("f''");
f(x)''
1/( (2^(1/2))^x + (2^(-1/2))^x )^3 * -1 * -2 * log(2) * (  log(2^(1/2))*(2^(1/2))^x + log(2^(-1/2))*(2^(-1/2))^x )

  1/( (2^(1/2))^x + (2^(-1/2))^x )^3 * -1 * -2 * log(2) * (  log(2^(1/2))*(2^(1/2))^x ) \
+ 1/( (2^(1/2))^x + (2^(-1/2))^x )^3 * -1 * -2 * log(2) * (  log(2^(-1/2))*(2^(-1/2))^x )

  (2^(1/2))^x /( (2^(1/2))^x + (2^(-1/2))^x )^3 * -1 * -2 * log(2) * log(2^(1/2))   \
+ (2^(-1/2))^x /( (2^(1/2))^x + (2^(-1/2))^x )^3 * -1 * -2 * log(2) * log(2^(-1/2))

  1/( (2^(-1/6))^x * (2^(1/2))^x +  (2^(-1/6))^x * (2^(-1/2))^x )^3 * -1 * -2 * log(2) * log(2^(1/2))   \
+ 1/( (2^(1/6))^x * (2^(1/2))^x + (2^(1/6))^x * (2^(-1/2))^x )^3 * -1 * -2 * log(2) * log(2^(-1/2))

  1/( (2^(-1/6) * 2^(1/2))^x  +  (2^(-1/6)*2^(-1/2))^x )^3 * -1 * -2 * log(2) * log(2^(1/2))   \
+ 1/( (2^(1/6) * 2^(1/2))^x   + (2^(1/6) * 2^(-1/2))^x )^3 * -1 * -2 * log(2) * log(2^(-1/2))

f2(x) = \
  1/( (2^(1/2-1/6) )^x  +  (2^(-1/6-1/2))^x )^3 * -1 * -2 * log(2) * log(2^(1/2))   \
+ 1/( (2^(1/2+1/6) )^x   + (2^( 1/6-1/2))^x )^3 * -1 * -2 * log(2) * log(2^(-1/2))
f2(x)
f2(0)


quit

rec_by_powers(x) = sum(i=0,100, 1/2*(-log(2))^i/i! *x^i);
f(3.0)
rec_by_powers(3)
quit

\\-----------------------------------------------------------------------------

log(16)
log(16)^2/2
log(16)^3/6
f = 1/(16^x);
f
print("f'");
f'
\\ d/dx 1/16^x
1/(16^x)^2 * -1 * log(16) * 16^x
1/(16^x) * -1 * log(16)

print("f''");
f''
1/(16^x) * -1 * -1 * log(16)^2

\\ 1/16^x = 1 - log(16)*x + log(16)^2/2*x^2 - log(16)^3/6*x^3

\\ Taylor f(x) = f'(0) + f'(0)*x + f''(0)*x^2 + ... + fn(0)/n!*x^n + R(x)
\\                     x
\\ Rn+1(x) =  1/n^2 * int  fn+1(t) * (x-n)^n dt
\\                     0
rec_by_powers(x) = sum(i=0,500, (-log(16))^i/i! *x^i);
1/16^3.0
rec_by_powers(3)
quit

