\\ Copyright 2014, 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("dragon-defines.gp");
read("../tools/memoize.gp");

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)=="t_COMPLEX" || type(y)=="t_COMPLEX",
     my(d=differ(real(x),real(y)));
     if(d != 0, return(Strprintf("real %s",d)));
     d=differ(imag(x),imag(y));
     if(d != 0, return(Strprintf("imag %s",d)));
     return(0));

  if(type(x)=="t_VEC" || type(x)=="t_COL",
     if(length(x)!=length(y),
        return(Strprintf("different lengths %s %s",length(x),length(y))));
     for(i=1,#x,
        my(d=differ(x[i],y[i]));
        if(d != 0, return(Strprintf("at index %d %s",i,d))));
     return(0));

  if(! nearly_equal(x,y),
     return(Strprintf("different values\ngot  %s\nwant %s",x,y)));
  return(0);
}

check_equal(got,want,name="") =
{
  check_count++;
  my(d = differ(got,want));
  if(d != 0,
     print("wrong ",name,"\n",d); quit(1),
     print("ok ",check_count," ",name));
}

\\-----------------------------------------------------------------------------

\\ TR1(0)==1 right sides
\\ TR1,2(0)==1 both sides
TR1(k) = if(k==0,1, TR1(k-1) + TR2(k-1) + TR3(k-1) + TR8(k-1));
TR2(k) = if(k==0,1, TR9(k-1) + TR10(k-1) + TR15(k-1));
TR3(k) = if(k==0,0, TR11(k-1));
TR4(k) = if(k==0,0, TR12(k-1));
TR5(k) = if(k==0,0, TR13(k-1));
TR6(k) = if(k==0,0, TR14(k-1));
TR7(k) = if(k==0,0, TR15(k-1));
TR8(k) = if(k==0,0, TR2(k-1) + TR3(k-1) + TR4(k-1) + TR5(k-1));
TR9(k) = if(k==0,0, TR4(k-1) + TR5(k-1) + TR6(k-1) + TR7(k-1));
TR10(k) = if(k==0,0, TR1(k-1) + TR6(k-1) + TR7(k-1) + TR8(k-1));
TR11(k) = if(k==0,0, TR9(k-1) + TR10(k-1));
TR12(k) = if(k==0,0, TR11(k-1));
TR13(k) = if(k==0,0, TR12(k-1));
TR14(k) = if(k==0,0, TR13(k-1));
TR15(k) = if(k==0,0, TR14(k-1));
TR1 = memoize(TR1);
TR2 = memoize(TR2);
TR8 = memoize(TR8);
TR9 = memoize(TR9);
TR10 = memoize(TR10);
TR11 = memoize(TR11);

TR_func_vector = [TR1,TR2,TR3,TR4,TR5,TR6,TR7,TR8, \
                  TR9,TR10,TR11,TR12,TR13,TR14,TR15];
check_exact(#TR_func_vector,15)
TR(k,j) = TR_func_vector[j](k);

{
  for(j=1,15, printf("%5d", j) );
  print();

  for(k=0,20,
     for(j=1,15,
        printf("%5d", TR(k,j));
        );
     print());
}

{
  for(j=1,15,
     print1("TR"j" ");
     for(k=0,12,print1(TR(k,j),","));
     print());
}

\\ big print
{
  my(j=10);
  print("TR"j);
  for(k=0,50,print1(TR(k,j),","));
  print();
}

TR_all_regions(k) = sum(j=1,15, TR(k,j));
check_exact(vector(300,k, TR_all_regions(k)), \
            vector(300,k, TR_all_regions(0)*2^k))

\\            1  2  3  4  5  6  7  8  9 10 11 12 13 14 15
TR_matrix = [ 1, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0; /* 1 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1; /* 2 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0; /* 3 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0; /* 4 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0; /* 5 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0; /* 6 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1; /* 7 */ \
              0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0; /* 8 */ \
              0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0; /* 9 */ \
              1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0; /* 10 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0; /* 11 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0; /* 12 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0; /* 13 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0; /* 14 */ \
              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0 ]; /* 15 */ \
{
  for(s=1,15,
     print1("TR" s "(k) = ");
     for(j=1,15,
        if(TR_matrix[s,j], print1(" "j)));
     print();
  );
}

TR_by_matrix(k,j) = (TR_matrix^k * TR_initial)[j,1];
TR_initial = matrix(15,1,i,j, i==1 || i==2);
{
  for(j=1,15,
     check_exact(vector(10,k,k--; TR_by_matrix(k,j)), \
                 vector(10,k,k--; TR(k,j)),
                 Strprintf("j=%d",j)));
}
check_exact(matrix(100,15,k,j, TR_by_matrix(k,j)), \
            matrix(100,15,k,j, TR(k,j)));

TR_gf_column = ((matid(15) - TR_matrix*x)^-1 \
                * TR_initial)[,1]
{
  print("all");
  for(s=1,#TR_gf_column,
     print1("gTR" s "(x) = ");
     pol_ascending_print(pol_partial_fractions(TR_gf_column[s])));
}
check_exact(vector(1,a, gf_terms(TR_gf_column[a],100)), \
            vector(1,a, vector(100,k,my(k=k-1); TR(k,a))))
TR_by_gf_column(k,s) = gf_term(TR_gf_column[s], k);

for(j=1,15,print(denominator(TR_gf_column[j])));
\\ 2*x^5 - x^4 + 4*x^3 - 2*x^2 + 2*x - 1
TR_by_recurrence(k,j) = 
{
  if(k<25,return(TR(k,j)));
  if (j <= 2,
      return(
           2 * TR(k-1,j)
         - 2 * TR(k-2,j)
         + 4 * TR(k-3,j)
         -     TR(k-4,j)
         + 2 * TR(k-5,j)
      ));
  if (j==9,
      return(
           2 * TR(k-1,j)
         -     TR(k-2,j)
         + 2 * TR(k-3,j)
      ));
               TR(k-1,j)
         + 0 * TR(k-2,j)
         + 2 * TR(k-3,j)
         + 3 * TR(k-4,j)
         + 1 * TR(k-5,j)
         + 2 * TR(k-6,j);
}
\\ j=2  1,0,1,3,3,5,14,31,57,109,224,455,903,1797
\\ 2*903 - 2*455 + 4*224 - 109 + 2*57
{
  for(j=1,15,
     check_exact(vector(100,k,k--; TR_by_recurrence(k,j)), \
                 vector(100,k,k--; TR(k,j)),
                 Strprintf("TR_by_recurrence j=%d",j)));
}

TR1_by_powers(k) =
{
  if(k<4,return(TR1(k)));
  53/100 * 2^k
  + vector_modulo([58/25, -9/25, -58/25, 9/25],k)
  + vector_modulo([-3/5, 4/5, 3/5, -4/5],k)*floor((k+2)/2)
  ;
}
check_exact(vector(30,k,k--; TR1_by_powers(k)), \
            vector(30,k,k--; TR(k,1)), \
            Strprintf("TR1_by_powers"));

TR1_by_powers(k) =
{
  if(k<4,return(TR1(k)));
  53/100 * 2^k
  + vector_modulo([-3/10, 2/5, 3/10, -2/5],k)*k
  + vector_modulo([43/25, 1/25, -43/25, -1/25],k)
  ;
}
check_exact(vector(30,k,k--; TR1_by_powers(k)), \
            vector(30,k,k--; TR(k,1)), \
            Strprintf("TR1_by_powers"));

\\ cf start 1,2
\\ -5/4 + 1/2*x + 2*x^2 + x^3
\\  + (53/100)/(1 - 2*x)
\\  + (58/25 - 9/25*x)/(1 + x^2)
\\  + (-3/5 + 4/5*x)/(1 + 2*x^2 + x^4)

gTR1(x) =
{
  -5/4 + 1/2*x + 2*x^2 + x^3
  + (53/100)/(1 - 2*x)
  + (58/25 - 9/25*x)/(1 + x^2)
  + (-3/5 + 4/5*x)/(1 + 2*x^2 + x^4)
  ;
}
check_exact(gf_terms(gTR1(x),100), vector(100,k,k--; TR1(k)))

\\---------
gTR3(x) = \
    8/75 * 1/(1-2*x) \
  + 1/12 * 1/(1+x) \
  - 1/100 * (89 + 3*x)/(1+x^2) \
  + 1/10 * (7 - x)/(1+x^2)^2;
check_exact(gf_terms(gTR3(x),100), vector(100,k,k--; TR3(k)))

TR3_by_powers(k) =
{
  if(k<4,return(TR3(k)));
  8/75 * 2^k
  + 1/12 * (-1)^k
  - vector_modulo([89/100, 3/100, -89/100, -3/100],k)
  + vector_modulo([7/10,-1/10,-7/10,1/10],k)*floor((k+2)/2)
  ;
}
check_exact(vector(30,k,k--; TR3_by_powers(k)), \
            vector(30,k,k--; TR(k,3)), \
            "TR3_by_powers from gf");

TR3_by_powers(k) =
{
  if(k<4,return(TR3(k)));
  8/75 * 2^k
  + vector_modulo([7/20, -1/20, -7/20, 1/20],k)*k
  + vector_modulo([-8/75, -49/300, 41/150, -1/300],k)
  ;
}
check_exact(vector(30,k,k--; TR3_by_powers(k)), \
            vector(30,k,k--; TR(k,3)), \
            "TR3_by_powers");

\\---------
gTR8(x) = \
  61/300*1/(1-2*x) \
  - 1/12 * 1/(1+x) \
  + 1/100*(53-169*x)/(1+x^2) \
  + 1/10*(1 + 7*x)/(1+x^2)^2 \
  - 3/4 + 3/2*x - x^3;
check_exact(gf_terms(gTR8(x),100), vector(100,k,k--; TR(k,8)))

TR8_by_powers(k) =
{
  if(k<4,return(TR8(k)));
  61/300 * 2^k
  - 1/12 * (-1)^k
  + vector_modulo([53/100, -169/100, -53/100, 169/100],k)
  + vector_modulo([1/10,7/10,-1/10,-7/10],k)*floor((k+2)/2)
  ;
}
check_exact(vector(30,k,k--; TR8_by_powers(k)), \
            vector(30,k,k--; TR(k,8)), \
            "TR8_by_powers");

TR8_by_powers(k) =
{
  if(k<4,return(TR8(k)));
  61/300 * 2^k
  + vector_modulo([1/20, 7/20, -1/20, -7/20],k)*k
  + vector_modulo([41/75, -377/300, -107/150, 427/300],k)
  ;
}
check_exact(vector(30,k,k--; TR8_by_powers(k)), \
            vector(30,k,k--; TR(k,8)), \
            "TR8_by_powers");

\\---------
gTR10(x) = \
    -3/4 - 1/2*x + x^2 + x^3 \
    + (113/300)/(1 - 2*x) \
    + (1/12)/(1 + x) \
    + (99/100 + 73/100*x)/(1 + x^2) \
    + (-7/10 + 1/10*x)/(1 + 2*x^2 + x^4);
check_exact(gf_terms(gTR10(x),100), vector(100,k,k--; TR10(k)))

TR10_by_powers(k) =
{
  if(k<4,return(TR10(k)));
  113/300 * 2^k
  + 1/12 * (-1)^k
  + vector_modulo([99/100, 73/100, -99/100, -73/100],k)
  + vector_modulo([-7/10,1/10,7/10,-1/10],k)*floor((k+2)/2)
  ;
}
check_exact(vector(30,k,k--; TR10_by_powers(k)), \
            vector(30,k,k--; TR(k,10)), \
            "TR10_by_powers from gf");

TR10_by_powers(k) =
{
  if(k<4,return(TR10(k)));
  113/300 * 2^k
  + vector_modulo([-7/20, 1/20, 7/20, -1/20],k)*k
  + vector_modulo([28/75, 209/300, -31/150, -259/300],k)
  ;
}
check_exact(vector(30,k,k--; TR10_by_powers(k)), \
            vector(30,k,k--; TR(k,10)), \
            "TR10_by_powers");


\\---------
gTR11(x) = \
    (16/75)/(1 - 2*x) \
  + (-1/12)/(1 + x) \
  + (-3/100 + 19/100*x)/(1 + x^2) \
  + (-1/10 - 7/10*x)/(1 + 2*x^2 + x^4);
check_exact(gf_terms(gTR11(x),100), vector(100,k,k--; TR11(k)))

TR11_by_powers(k) =
{
  if(k<4,return(TR11(k)));
  16/75 * 2^k
  + -1/12 * (-1)^k
  + vector_modulo([-3/100, 19/100, 3/100, -19/100],k)
  + vector_modulo([-1/10,-7/10,1/10,7/10],k)*floor((k+2)/2)
  ;
}
check_exact(vector(30,k,k--; TR11_by_powers(k)), \
            vector(30,k,k--; TR(k,11)), \
            "TR11_by_powers from gf");

TR11_by_powers(k) =
{
  if(k<4,return(TR11(k)));
  16/75 * 2^k
  + vector_modulo([-16/75, -23/300, 7/150, 73/300],k)
  + vector_modulo([-1/20, -7/20, 1/20, 7/20],k)*k
  ;
}
check_exact(vector(30,k,k--; TR11_by_powers(k)), \
            vector(30,k,k--; TR(k,11)), \
            "TR11_by_powers");


quit
\\-----------------------------------------------------------------------------

\\   4      2      5
\\       ------->
\\   3      1      6

QR1(k) = TR1(k) + TR10(k) + TR8(k) + TR9(k);
QR2(k) = TR2(k) + TR15(k) + TR3(k);
QR5(k) = TR4(k) + TR5(k);
QR6(k) = TR6(k) + TR7(k);
QR3(k) = TR11(k) + TR12(k);
QR4(k) = TR13(k) + TR14(k);

QR_func_vector = [QR1,QR2,QR3,QR4,QR5,QR6];
check_exact(#QR_func_vector,6)
QR(k,j) = QR_func_vector[j](k);

QR_all_regions(k) = QR1(k) + QR2(k) + QR3(k) + QR4(k) + QR5(k) + QR6(k);
check_exact(vector(300,k, QR_all_regions(k)), vector(300,k, 2*2^k))
{
  print("QR");
  print1("      ");
  for(j=1,6, printf("%7d", j) );
  print();

  for(k=0,20,
     printf("k=%2d  ", k);
     for(j=1,6,
        printf("%7d", QR(k,j));
        );
     print());
}
print("QR1");
for(k=0,50,print1(QR1(k),","))

gQR1(x) =
{
     + 1/2 + x
     + 29/50* 1/(1 - 2*x)
     + 1/50*(-7 - 24*I)/(1 + I*x)
     + 1/10*(1 + 2*I)/(1 + 2*I*x - x^2)
     + 1/50*(-7 + 24*I)/(1 - I*x)
     + 1/10*(1 - 2*I)/(1 - 2*I*x - x^2);
}


\\-----------------------------------------------------------------------------
\\ QR by own recurrences

\\   4      2      5
\\       ------->
\\   3      1      6

\\
\\       |                     |
\\      3 4      6 5 6   1       3
\\---                <---------      --------
\\       |       5  ^    2       4
\\       |          |          | 
\\      1|2        2|1         |
\\       |          |          
\\    6  v    1     |  3
\\        <--------- 
\\    5       2        4
\\
\\      6|5        4 3
\\       |

QR1(k) =
{
  if(k==0,return(1));
  if(k==1,return(4));
  2*QR1(k-2) \
       + 4*QR2(k-2) \
       + 2*QR3(k-2) \
       + 2*QR4(k-2) \
       + 2*QR5(k-2) \
       ;
}
QR2(k) =
{
  if(k==0,return(1));
  if(k==1,return(0));
  QR1(k-2)         + QR4(k-2) +   QR5(k-2) + 2*QR6(k-2);
}
QR3(k) =
{
  if(k<=1,return(0));
  QR1(k-2)                    +   QR5(k-2) + 2*QR6(k-2);
}
QR4(k) =
{
  if(k<=1,return(0));
          QR3(k-2);
}
QR5(k) =
{
  if(k<=1,return(0));
          QR3(k-2);
}
QR6(k) = 
{
  if(k<=1,return(0));
                   QR4(k-2);
}
QR1 = memoize(QR1);
QR2 = memoize(QR2);
QR3 = memoize(QR3);

QR_matrix = [ 2, 4, 2, 2, 2, 0; \
              1, 0, 0, 1, 1, 2; \
              1, 0, 0, 0, 1, 2; \
              0, 0, 1, 0, 0, 0; \
              0, 0, 1, 0, 0, 0; \
              0, 0, 0, 1, 0, 0]
polroots(charpoly(QR_matrix))
check_exact(sum(i=1,6, sum(j=1,6, QR_matrix[i,j])), 4*6, "matrix sum")

QR_func_vector = [QR1,QR2,QR3,QR4,QR5,QR6];
check_exact(#QR_func_vector,6)
QR(k,j) = QR_func_vector[j](k);

QR_initial_even = [1;1;0;0;0;0];
QR_initial_odd  = [4;0;0;0;0;0];
QR_by_matrix(k,j) = (QR_matrix^floor(k/2) \
                     * if(k%2==0, QR_initial_even, QR_initial_odd))[j,1];
check_exact(matrix(200,6,k,j, QR_by_matrix(k,j)), \
            matrix(200,6,k,j, QR(k,j)));

{
  print("QR");
  print1("      ");
  for(j=1,6, printf("%7d", j) );
  print();

  for(k=0,20,
     printf("k=%2d  ", k);
     for(j=1,6,
        printf("%7d", QR(k,j));
        );
     print());
}

QR_gf_column_even = ((matid(6) - QR_matrix*x)^-1 * QR_initial_even)[,1];
{
  print("even");
  for(j=1,#QR_gf_column_even,
     pol_ascending_print(pol_partial_fractions(QR_gf_column_even[j])));
}
QR_gf_column_odd = ((matid(6) - QR_matrix*x)^-1 * QR_initial_odd)[,1];
{
  print("odd");
  for(j=1,#QR_gf_column_odd,
     pol_ascending_print(pol_partial_fractions(QR_gf_column_odd[j])));
}

\\ G - x^2*G*M = Initial
\\ G = (1 - x^2*M)^-1 * Initial
\\
QR_gf_column = ((matid(6) - QR_matrix*x^2)^-1 \
                * (QR_initial_even + x*QR_initial_odd))[,1];
{
  print("all");
  for(s=1,#QR_gf_column,
     print1("gQR" s "(x) = ");
     pol_ascending_print(pol_partial_fractions(QR_gf_column[s], I)));
}
check_exact(vector(1,a, gf_terms(QR_gf_column[a],100)), \
            vector(1,a, vector(100,k,my(k=k-1); QR(k,a))))
QR_by_gf_column(k,s) = gf_term(QR_gf_column[s], k);

{
  print("QR_by_gf_column");
  print1("      ");
  for(j=1,6, printf("%7d", j) );
  print();

  for(k=16,16,
     printf("k=%2d  ", k);
     for(j=1,6,
        printf("%7d", QR_by_gf_column(k,j));
        );
     print());
}
{
  for(k=2,20,
     print(QR1(k)/2^k * 1.0));
}
{
  for(k=2,20,
     print(gf_term(QR_gf_column[1],k) /2^k * 1.0));
}
