// Copyright 2015, 2016 Kevin Ryde
//
// This file is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 3, or (at your option) any later
// version.
//
// This file is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this file.  See the file COPYING.  If not, see
// <http://www.gnu.org/licenses/>.


// Draw the dragon curve of given "level".
//
// dragon() draws along the X axis 0,0 to 1,0.  The recursion draws
// sub-curves from those ends inward to the right-side middle at point
// (.5, -.5).
//
//     0,0          1,0
//        \        /
//         \      /
//          .5,-.5
//
// The two transformation matrices m1 and m2 reduce, rotate and shift
// the sub-curves for this.  They're the same as explicit rotates in
// dragon.scad but split out.

level = 8;
linewidth = .1;  // fraction of segment length
$fs = .05;       // smooth the sphere at curve start

sqrt2 = pow(2, .5);
m1 = [ [ .5, .5,     0,   0 ],   // rotate -45deg
       [-.5, .5,     0,   0 ],   // and shrink all x,y,z by 1/sqrt(2)
       [  0,  0, 1/sqrt2, 0 ],
       [  0,  0,     0,   1 ]
     ];
m2 = [ [-.5,  .5,     0,   1 ],  // rotate -135deg, shift to 1,0
       [-.5, -.5,     0,   0 ],  // and shrink all x,y,z by 1/sqrt(2)
       [  0,   0, 1/sqrt2, 0 ],
       [  0,   0,     0,   1 ]
     ];

// Draw a dragon curve "level" going from 0,0 to 1,0
module dragon(level) {
    if (level <= 0) {
        translate([.5,0]) cube([1+linewidth,linewidth,linewidth],center=true);
    } else {
        multmatrix(m1) dragon(level-1);
        multmatrix(m2) dragon(level-1);
    }
}

// scale up so it's nicely visible in the default openscad GUI
scale(40) {
    sphere(1.5*linewidth / pow(2,level/2));  // mark the start of the curve
    dragon(level);
}


//-----------------------------------------------------------------------------
// Alternative code for segment drawn by extrude of polygon instead of cube().
// Could extrude everything at the toplevel instead too.
//
// linear_extrude(center=true, height=2*linewidth)
// polygon([ [ -linewidth,linewidth], [-linewidth,-linewidth],
//           [1+linewidth,-linewidth], [1+linewidth,linewidth] ]);


// Local variables:
// compile-command: "openscad --imgsize=1000,800 -o /tmp/dragon.png dragon-matrix.scad && xzgv /tmp/dragon.png"
// End:
