#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;

use FindBin;
use lib $FindBin::Bin;
use MyDragon '@dir4_to_dx','@dir4_to_dy';
use MyPrintwrap ':all';


my $type = $ARGV[0] // 'BP1-a';
print "  % generated by tools/$FindBin::Script $type\n";

my %type_to_segments
  = ('BP1-a' => ['0,0,0','0,0,1',
                 '1,1,0','1,1,1','1,1,2','1,1,3'],

     'BP1-b' => ['0,0,2','0,0,1',
                 '-1,1,0',
                 '1,1,2',
                 '0,2,0','0,2,2','0,2,3'],

     'BP1-p' => ['0,0,0','0,0,1',
                 '1,1,0','1,1,1','1,1,2','1,1,3',
                 '2,2,1','2,2,2','2,2,3',
                 '1,3,0','1,3,1',
                 '3,3,2',
                 '2,4,0','2,4,1','2,4,2','2,4,3'],

     'BP6-a' => ['0,0,1','-1,1,0',
                 '1,1,2','1,1,1',
                 '0,2,0','0,2,3'],

    );


my $segs = $type_to_segments{$type};
$Printwrap = 2;
foreach my $seg (@$segs) {
  MyDragonConf->seg_validate($seg);
  my ($x,$y,$dir) = split /,/, $seg;
  my $x2 = $x + $dir4_to_dx[$dir];
  my $y2 = $y + $dir4_to_dy[$dir];
  printwrap(" \\draw[my arrow] ($x,$y)--($x2,$y2);");
}
printwrap_newline();

print "  \\begin{scope}[shift=(pos-1)]\n";
foreach my $seg (@$segs) {
  MyDragonConf->seg_validate($seg);
  my ($x,$y,$dir) = split /,/, $seg;
  my $dx = $dir4_to_dx[$dir];
  my $dy = $dir4_to_dy[$dir];
  printwrap(" \\pic at ($x,$y) [transform shape] {my expand pic={$dx,$dy}};");
}
printwrap_newline();
print "  \\end{scope}\n";

print "  % end generated\n";
exit 0;
