#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min';
use Math::BaseCnv;
use Math::BigInt;

use FindBin;
use lib $FindBin::Bin;
use MyDragon '@MinusRot_state_table','@MinusRot_state_to_dz','xy_to_gp_str';
use MyPrintwrap ':all';
use MyFLAT;
$|=1;

print "% generated by tools/$FindBin::Script\n";
my $max_len = Math::BigInt->new(32);

printwrap_indent("% GP-DEFINE  ");
my @dz = map {xy_to_gp_str(@$_)} @MinusRot_state_to_dz;
printwrap_gp_vector(\@dz, "MinusRot_state_to_dz");

{
  my $prefix = "MinusRot_state_table = {[";
  foreach my $state (0 .. $#MinusRot_state_table) {
    my $semi = $state == $#MinusRot_state_table ? ' ]};' : ';   ';
    printf "%% GP-DEFINE  %s %2d,%2d%s  \\\\ %2d  %s\n",
      $prefix,
      $MinusRot_state_table[$state]->[0] + 1,
      $MinusRot_state_table[$state]->[1] + 1,
      $semi,
      $state + 1,
      $dz[$state];
    $prefix = ' ' x length($prefix);
  }
}

print "% end generated\n";
exit 0;
