#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','transform_to_horizontal','chamfer';
use MyPrintwrap ':all';


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
{
  my ($x,$y) = transform($x_hi,$y_hi);
  print "  \\node (connect) at ($x,$y) [my blank dot];\n";
}

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level-2);
}

foreach my $other (0,1) {
  $Printwrap = 2;
  printwrap("\\draw[->] ");
  $Printwrap = 4;
  my $join = '';
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    if ($other) {
      ($x,$y) = xy_rotate_plus90($x,$y);
      ($x,$y) = ($x+$x_hi, $y+$y_hi);
      ($x,$y) = ($x+$y_hi, $y-$x_hi);
    }
    ($x,$y) = transform($x,$y);
    if ($n == $n_hi) {
      printwrap("$join(connect)");
    } elsif ($n == ($other ? 7 : 5)) {
      my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
      my ($tdx,$tdy) = $path->n_to_dxdy($n);
      ($sdx,$sdy) = transform($sdx,$sdy);
      ($tdx,$tdy) = transform($tdx,$tdy);
      if ($other) {
        ($sdx,$sdy) = xy_rotate_plus90($sdx,$sdy);
        ($tdx,$tdy) = xy_rotate_plus90($tdx,$tdy);
      }
      my $sx = $x . chamfer(-$sdx);
      my $sy = $y . chamfer(-$sdy);
      my $tx = $x . chamfer($tdx);
      my $ty = $y . chamfer($tdy);
      printwrap("$join($sx,$sy)");
      printwrap("--($tx,$ty)");
    } else {
      printwrap("$join($x,$y)");
    }
    $join = '--';
  }
  printwrap(";\n");
}

print "  % end generated\n";
exit 0;
