#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Graph::Maker::TwindragonAreaTree;
use Math::PlanePath::ComplexPlus;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'TA_complex_c';


my $max_level = 6;
if (@ARGV) { $max_level = shift @ARGV; }

print "% generated by tools/$FindBin::Script $max_level\n";

my $var = $FindBin::Script;
$var =~ s/-TAW\.pl$//;
$var =~ tr/-/_/;

my %arrays;
foreach my $k (0 .. $max_level) {
  my $graph = Graph::Maker->new ('twindragon_area_tree',
                                 level => $k,
                                 undirected => 1);
  my $Sv = '0';
  my $Cv = TA_complex_c($k);
  $graph->has_vertex($Sv) or die "k=$k";
  $graph->has_vertex($Cv) or die "k=$k no Cv=$Cv";

  my $TAW = 0;
  my $TAwS = 0;
  my $TAwC = 0;
  my $count_pairs = 0;
  my $count_distinct_pairs = 0;
  my $for = $graph->for_shortest_paths
    (sub {
       my ($t, $u,$v, $n) = @_;
       if ($u le $v) {   # one direction only
         $TAW += $t->path_length($u,$v);
         $count_pairs++;
         if ($u lt $v) { $count_distinct_pairs++; }
       }
       if ($u eq $Sv) {
         $TAwS += $t->path_length($u,$v);
       }
       if ($u eq $Cv) {
         $TAwC += $t->path_length($u,$v);
       }
     });
  push @{$arrays{'TAW'}},   $TAW;
  push @{$arrays{'TAwS'}},  $TAwS;
  push @{$arrays{'TAwC'}},  $TAwC;
  push @{$arrays{'TAW_count_pairs'}}, $count_pairs;
  push @{$arrays{'TAW_count_distinct_pairs'}}, $count_distinct_pairs;
}

foreach my $name ('TAW', 'TAW_count_pairs', 'TAW_count_distinct_pairs',
                  'TAwS', 'TAwC', ) {
  my $aref = $arrays{$name};
  print "% GP-DEFINE  gen_${name}_samples = [", join(',',@$aref),"];\n";
}

print "% end generated\n";
exit 0;
