#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 8;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $segments = MyLineSegments->new;
foreach my $rotate (0, 1) {
  {
    my ($x,$y) = ($x_hi,$y_hi);
    if ($rotate) { ($x,$y) = (-$x,-$y); } # rotate 180
    print "  \\node (end-$rotate) at ($x,$y) [my dot];\n";
  }
  foreach my $n ($n_lo .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    $x = $x_hi - $x;
    $y = $y_hi - $y;
    my ($x2,$y2) = $path->n_to_xy($n+1);
    $x2 = $x_hi - $x2;
    $y2 = $y_hi - $y2;

    if ($rotate) { ($x,$y) = (-$x,-$y); } # rotate 180
    if ($rotate) { ($x2,$y2) = (-$x2,-$y2); } # rotate 180
    $segments->add($x,$y, $x2,$y2);
  }
}
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

print "  % end generated\n";
exit 0;
