#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyLineSegments;
use MyPrintwrap ':all';

# 2 8
# 3 8
# 4 5
my $arms = 2;
my $level = 8;

if (@ARGV) {
  @ARGV == 2 or die;
  ($arms,$level) = @ARGV;
}

print "  % generated by tools/$FindBin::Script $arms $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new (arms => $arms);
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-$arms) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n + $arms));
}
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
my $end = 0;
foreach my $n ($n_lo, $n_hi-($arms-1) .. $n_hi) {
  my $name = ($n == 0 ? "origin" : ($end++, "end-$end"));
  my ($x,$y) = $path->n_to_xy($n);
  printwrap("\\node ($name) at ($x,$y) [my dot];  % N=$n\n");
}

my @segs = $segments->segments;
my @L_boundary_segs = grep {! $segments->is_surrounded_L($_) } @segs;
my @R_boundary_segs = grep {! $segments->is_surrounded_R($_) } @segs;

print "% GP-Test-Last  Ba$arms($level) == ",
  scalar(@L_boundary_segs) + scalar(@R_boundary_segs),
  "\n";
print "  % end generated\n";
exit 0;
