#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve;

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'BlobN','xy_rotate_plus90','xy_rotate_minus90';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

my $n_lo = BlobN($level);
my $n_hi = BlobN($level+1) - 3;

print "  \\newcommand{\\MyLevel}{$level}\n";

$Printwrap = 2;
{
  my ($x_lo,$y_lo) = $path->n_to_xy($n_lo);
  printwrap("\\coordinate (start) at ($x_lo,$y_lo);\n");
}
{
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  printwrap("\\coordinate (end) at ($x_hi,$y_hi);\n");
}

sub n_is_right_boundary {
  my ($n) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_minus90($dx,$dy);  # to right side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! defined $path->xyxy_to_n_either($x1,$y1, $x1s,$y1s)
          || ! defined $path->xyxy_to_n_either($x2,$y2, $x2s,$y2s)
          || ! defined $path->xyxy_to_n_either($x1s,$y1s, $x2s,$y2s));
}

sub n_is_left_boundary {
  my ($n) = @_;
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to left side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! defined $path->xyxy_to_n_either($x1,$y1, $x1s,$y1s)
          || ! defined $path->xyxy_to_n_either($x2,$y2, $x2s,$y2s)
          || ! defined $path->xyxy_to_n_either($x1s,$y1s, $x2s,$y2s));
}

my $BlobL = 0;
my $BlobR = 0;

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
  if (n_is_left_boundary($n)) { $BlobL++; }
  if (n_is_right_boundary($n)) { $BlobR++; }
}

{
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

print "  \\newcommand{\\MyBlobL}{$BlobL}\n";
print "  \\newcommand{\\MyBlobR}{$BlobR}\n";
print "% GP-Test-Last  BlobL($level)==$BlobL\n";
print "% GP-Test-Last  BlobR($level)==$BlobR\n";

print "  % end generated\n";
exit 0;
