#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my @north = (0,1);
my @south = (0,-1);
my @east = (1,0);
my @west = (-1,0);

my @ER_to_OR = ([0,0, @north],   # ER bottom
                [0,0, @south],
                [0,0, @east],
                [0,0, @west],
                [-1,-1, @north],
                [1,-1, @north],
                [1,-1, @west],
                [0,-2, @north],

                [-1,1, @south],  # part of hanging square

                [1,1, @west],    # bridge
                [1,1, @north],

                [0,2, @north],   # OR top
                [0,2, @east],
                [1,3, @south],
                [2,2, @north],
                [2,2, @west],
               );
my @ER_to_OR_boundary_points = ([0,2], [2,2],  # OR sides
                                [-1,1], [0,-1], # ER across
                               );

my @EL_to_OL = ([0,0, @north],   # EL bottom
                [0,0, @south],
                [0,0, @east],
                [0,0, @west],
                [-1,-1, @north],
                [1,-1, @north],

                [1,1, @south],  # part of hanging square

                [-1,1, @east],    # bridge
                [-1,1, @north],

                [-2,2, @north],   # OL top
                [-2,2, @east],
                [-1,3, @north],
                [-1,3, @south],
                [-1,3, @east],
                [0,2, @north],
                [0,2, @west],
               );
my @EL_to_OL_boundary_points = ([-2,2], [0,3],  # OL sides
                                [-1,-1], [1,1], # EL across
                               );

foreach my $elem ([\@ER_to_OR, \@ER_to_OR_boundary_points, "ER-to-OR"],
                  [\@EL_to_OL, \@EL_to_OL_boundary_points, "EL-to-OL"],
                 ) {
  my ($segs, $points, $name) = @$elem;

  my %arrows_end;
  foreach my $seg (@$segs) {
    my ($x,$y, $dx,$dy) = @$seg;
    my ($x2,$y2) = ($x + $dx, $y + $dy);
    ($x+$y) % 2 == 0 or die;
    $arrows_end{"$x2,$y2"}++;
  }

  print "  \\begin{scope}[shift=(pos-$name)]\n";
  $Printwrap = 4;
  foreach my $seg (@$segs) {
    my ($x,$y, $dx,$dy) = @$seg;
    my ($x2,$y2) = ($x + $dx, $y + $dy);
    my $line_type = ($arrows_end{"$x2,$y2"} >= 2 ? ',my line' : '');
    printwrap(" \\draw[->$line_type] ($x,$y)--+($dx,$dy);");
  }
  printwrap_newline();
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    printwrap(" \\node at ($x,$y) [my boundary dot];");
  }
  printwrap_newline();
  print "  \\end{scope}\n";

  print "  \\begin{scope}[shift=(pos-$name-expand)]\n";
  $Printwrap = 4;
  foreach my $seg (@$segs) {
    my ($x,$y, $dx,$dy) = @$seg;
    printwrap(" \\pic at ($x,$y) [transform shape] {my expand pic={$dx,$dy}};");
  }
  printwrap_newline();
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    printwrap(" \\node at ($x,$y) [my boundary dot];");
  }
  printwrap_newline();
  print "  \\end{scope}\n";
}

print "  % end generated\n";
exit 0;
