#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','rect_corners_sort',
  'transform_to_horizontal','chamfer',;
use MyPrintwrap ':all';
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 7;  # two k=7 joining to form k=8
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevelHalf}{",$level,"}\n";
print "  \\newcommand{\\MyLevelNew}{",$level+1,"}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_mid) = $path->level_to_n_range($level);

my ($xm, $ym) = $path->n_to_xy($n_mid);
my $n_hi = 2*$n_mid;
print "  % $n_lo to $n_mid to $n_hi\n";

sub transform {
  my ($x,$y) = @_;
  ($x,$y) = transform_to_horizontal($x,$y, $level+1);
  return ($x,$y);
}

$Printwrap = 2;
{
  my $n = $n_lo;
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (start) at ($x,$y) [my dot];\n");
}
{
  my $n = $n_mid;
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (middle) at ($x,$y) [my dot];\n");
}
{
  my $n = $n_hi;
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = transform($x,$y);
  printwrap("\\node (end) at ($x,$y) [my dot];\n");
}

sub xy_is_join {
  my ($x,$y) = @_;
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ <= $n_hi} @n_list;  # within $level
  return (@n_list > 1
          && $n_list[0] < $n_mid
          && $n_list[1] > $n_mid);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
my $join = '';
my %seen;
my @squares;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  ### at: "n=$n  $x,$y"
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  my $is_join = xy_is_join($x,$y);

  ($x,$y) = transform($x,$y);
  ($dx,$dy) = transform($dx,$dy);

  if ($is_join) {
    my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
    ($sdx,$sdy) = transform($sdx,$sdy);

    my $sx = $x . chamfer(-$sdx);
    my $sy = $y . chamfer(-$sdy);
    my $tx = $x . chamfer($dx);
    my $ty = $y . chamfer($dy);
    printwrap("$join($sx,$sy)");
    printwrap("--($tx,$ty)");

    if ($n < $n_mid) {
      my ($ldx,$ldy) = xy_rotate_plus90($dx,$dy);
      my $x1 = $x;
      my $y1 = $y;
      my $x2 = $x + $dx + $ldx;
      my $y2 = $y + $dy + $ldy;
      ($x1,$y1, $x2,$y2) = rect_corners_sort ($x1,$y1, $x2,$y2);
      next if $seen{"$x1,$y1"}++;

      push @squares, "($x1,$y1)";
    }

  } else {
    printwrap("$join($x,$y)");
  }
  $join = '--';
}
printwrap(";\n");

{
  $Printwrap = 2;
  printwrap("\\fill[my square colour] foreach \\p in {");
  $Printwrap = 4;
  foreach my $str (@squares) {
    printwrap("$str,");
  }
  printwrap("}\n");
  printwrap("{ \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");
}

my $count_squares = scalar(@squares);
print "  \\newcommand{\\MyJoinArea}{$count_squares}\n";
print "% GP-Test  JA($level) == $count_squares\n";

print "  % end generated\n";
exit 0;
