#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use List::Util 'min','max';
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'chamfer';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 8;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
# print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo,$n_hi) = $path->level_to_n_range($level);

my $ab_length = $n_hi / 16;
my $a_start = 6*$ab_length;
my $a_end   = $a_start + $ab_length;
my $b_start = 12*$ab_length;
my $b_end   = $b_start + $ab_length;

{
  my ($x,$y) = $path->n_to_xy($a_start);
  print "  \\coordinate (start-a) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($a_end);
  print "  \\coordinate (end-a) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($b_start);
  print "  \\coordinate (start-b) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($b_end);
  print "  \\coordinate (end-b) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}
for (my $n = $n_lo; $n <= $n_hi; $n += $ab_length) {
  $segments->remove($path->n_to_xy($n),
                    $path->n_to_xy($n + $ab_length));
}
$Printwrap = 2;
printwrap("\\draw[my grey] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
for (my $n = $n_lo; $n < $n_hi; $n += $ab_length) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n + $ab_length);

  my $thick = ($n >= $a_start && $n < $a_end)
    || ($n >= $b_start && $n < $b_end);

  if (($n / $ab_length) % 2) {
    ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
  }
  printwrap(" \\draw"
            .($thick ? "[my thick] " : "[my thin] ")
            ."($x1,$y1)--($x2,$y2);");
}
printwrap_newline();

print "  % end generated\n";
exit 0;
