#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::ComplexMinus 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 5;
print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::ComplexMinus->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\newcommand{\\MyNhi}{$n_hi}\n";
print "  \\node (start) at (0,0) [my dot];\n";
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  $x *= 2;
  $y *= 2;
  print "  \\node (end) at ($x,$y) [my dot];\n";
}

{
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    $x *= 2;
    $y *= 2;
    $segments->add($x-1,$y-1, $x+1,$y-1); # horizontal
    $segments->add($x-1,$y+1, $x+1,$y+1);
    $segments->add($x-1,$y-1, $x-1,$y+1); # vertical
    $segments->add($x+1,$y-1, $x+1,$y+1);
  }
  $Printwrap = 2;
  printwrap("\\draw[my squares] ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

{
  $Printwrap = 2;
  printwrap("\\draw (0,0)");
  $Printwrap += 2;
  my $prev_x = 0;
  my $prev_y = 0;
  foreach my $n ($n_lo+1 .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    $x *= 2;
    $y *= 2;

    my $dx = $x - $prev_x;
    my $dy = $y - $prev_y;
    if ($dx==0 && $dy == -6) {
      printwrap("..controls +(\\MyUpperControl) and +(\\MyLowerControl)");
      printwrap("..($x,$y)");
    } else {
      printwrap("--($x,$y)");
    }

    ($prev_x,$prev_y) = ($x,$y);
  }
}
printwrap(";\n");

print "  % end generated\n";
exit 0;
