#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_rotate_plus90','xy_rotate_minus90';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 8;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::DragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my ($x_p3,$y_p3) = xy_rotate_plus90($x_hi,$y_hi);
my ($x_p2,$y_p2) = ($x_hi + $x_p3, $y_hi + $y_p3);

print "  \\node (start-0) at (0,0) [my dot];\n";
print "  \\node (start-1) at ($x_hi,$y_hi) [my dot];\n";
print "  \\node (start-2) at ($x_p2,$y_p2) [my dot];\n";
print "  \\node (start-3) at ($x_p3,$y_p3) [my dot];\n";

sub xyxy_cycle_0 {
  return @_;
}
sub xyxy_cycle_1 {
  my ($x,$y) = @_;
  ($x,$y) = xy_rotate_plus90($x,$y);
  return ($x + $x_hi, $y + $y_hi);
}
sub xyxy_cycle_2 {
  my ($x,$y) = @_;
  ($x,$y) = (-$x,-$y); # rotate 180
  return ($x + $x_p2, $y + $y_p2);
}
sub xyxy_cycle_3 {
  my ($x,$y) = @_;
  ($x,$y) = xy_rotate_minus90($x,$y);
  return ($x + $x_p3, $y + $y_p3);
}
my @cycle_funcs = (\&xyxy_cycle_0, \&xyxy_cycle_1,
                   \&xyxy_cycle_2, \&xyxy_cycle_3);

sub xyxy_uncycle_0 {
  return @_;
}
sub xyxy_uncycle_1 {
  my ($x,$y) = @_;
  $x -= $x_hi;
  $y -= $y_hi;
  return xy_rotate_minus90($x,$y);
}
sub xyxy_uncycle_2 {
  my ($x,$y) = @_;
  $x -= $x_p2;
  $y -= $y_p2;
  return (-$x,-$y);
}
sub xyxy_uncycle_3 {
  my ($x,$y) = @_;
  $x -= $x_p3;
  $y -= $y_p3;
  return xy_rotate_plus90($x,$y);
}
my @uncycle_funcs = (\&xyxy_uncycle_0, \&xyxy_uncycle_1,
                     \&xyxy_uncycle_2, \&xyxy_uncycle_3);

sub xyxy_is_cycle_touch {
  my ($x,$y) = @_;
  my $count = 0;
  foreach my $func (@uncycle_funcs) {
    my $n = $path->xy_to_n($func->($x,$y));
    $count += (defined $n && $n < $n_hi);
  }
  return $count >= 2;
}

my $segments = MyLineSegments->new;
foreach my $i (0 .. $#cycle_funcs) {
  my $func = $cycle_funcs[$i];
  my ($prev_x,$prev_y) = $func->(0,0);
  foreach my $n ($n_lo+1 .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    ($x,$y) = $func->($x,$y);
    $segments->add($x,$y, $prev_x,$prev_y);
    $prev_x = $x;
    $prev_y = $y;
  }
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

print "  % end generated\n";
exit 0;



__END__

# sub xyxy_uncycle_0 {
#   return @_;
# }
# sub xyxy_uncycle_1 {
#   my ($x,$y) = @_;
#   $x -= $x_hi;
#   $y -= $y_hi;
#   return xy_rotate_minus90($x,$y);
# }
# sub xyxy_uncycle_2 {
#   my ($x,$y) = @_;
#   $x -= $x_p2;
#   $y -= $y_p2;
#   return (-$x,-$y);
# }
# sub xyxy_uncycle_3 {
#   my ($x,$y) = @_;
#   $x -= $x_p3;
#   $y -= $y_p3;
#   return xy_rotate_plus90($x,$y);
# }
# my @uncycle_funcs = (\&xyxy_uncycle_0, \&xyxy_uncycle_1,
#                      \&xyxy_uncycle_2, \&xyxy_uncycle_3);
#
# sub xyxy_is_cycle_touch {
#   my ($x,$y) = @_;
#   my $count = 0;
#   foreach my $func (@uncycle_funcs) {
#     my $n = $path->xy_to_n($func->($x,$y));
#     $count += (defined $n && $n < $n_hi);
#   }
#   return $count >= 2;
# }
#
# foreach my $i (0 .. $#cycle_funcs) {
#   my $func = $cycle_funcs[$i];
#   $Printwrap = 2;
#   printwrap("\\draw (start-$i)");
#   $Printwrap = 4;
#   foreach my $n ($n_lo+1 .. $n_hi-1) {
#     my ($x,$y) = $path->n_to_xy($n);
#     ($x,$y) = $func->($x,$y);
#
#     if (xyxy_is_cycle_touch($x,$y)) {
#       my ($x0,$y0) = $path->n_to_xy($n-1);
#       ($x0,$y0) = $func->($x0,$y0);
#
#       my ($x2,$y2) = $path->n_to_xy($n+1);
#       ($x2,$y2) = $func->($x2,$y2);
#
#       my $sdx = $x - $x0;
#       my $sdy = $y - $y0;
#       my $tdx = $x2 - $x;
#       my $tdy = $y2 - $y;
#
#       my $sx = $x . chamfer(-$sdx);
#       my $sy = $y . chamfer(-$sdy);
#       my $tx = $x . chamfer($tdx);
#       my $ty = $y . chamfer($tdy);
#       printwrap("--($sx,$sy)");
#       printwrap("--($tx,$ty)");
#     } else {
#       printwrap("--($x,$y)");
#     }
#   }
#   my $i_next = ($i+1) % scalar(@cycle_funcs);
#   printwrap("--(start-$i_next);\n");
# }
