#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'xy_to_dir4','@dir4_to_dx','@dir4_to_dy';
use MyPrintwrap ':all';


my $level = 8;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

# {
#   $Printwrap = 2;
#   printwrap("\\path foreach \\p in {");
#   $Printwrap += 2;
#   my %seen;
#   foreach my $n ($n_lo .. $n_hi) {
#     my ($x,$y) = $path->n_to_xy($n);
#     next if $seen{"$x,$y"}++;
#     printwrap("($x,$y),");
#   }
#   printwrap("}");
#   printwrap(" { \\p node[my dot] };\n");
#   printwrap_newline();
# }

my @segments;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my $dir4 = xy_to_dir4($dx,$dy);
  push @{$segments[$dir4]}, "$x,$y";
}
printwrap_newline();

foreach my $dir4 (0 .. 3) {
  $Printwrap = 2;
  printwrap("\\path foreach \\p in {");
  $Printwrap += 2;
  foreach my $p (@{$segments[$dir4]}) {
    printwrap("($p),");
  }
  my $dx = $dir4_to_dx[$dir4];
  my $dy = $dir4_to_dy[$dir4];
  printwrap("}");
  printwrap("{ \\p pic[transform shape] {my L={$dx,$dy}} };\n");
}

print "  % end generated\n";
exit 0;
