#!/usr/bin/perl -w

# Copyright 2015, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_mid) = $path->level_to_n_range(3);

my $n_hi = 2*$n_mid;
print "  % $n_lo to $n_mid to $n_hi\n";

$Printwrap = 2;
my ($x_mid,$y_mid) = $path->n_to_xy($n_mid);
printwrap("\\node (middle) at ($x_mid,$y_mid) [my dot];\n");

{
  my ($x,$y) = $path->n_to_xy($n_hi);
  printwrap("\\coordinate (end) at ($x,$y);\n");
}

sub xy_str {
  my ($x,$y) = @_;
  if ($x eq $x_mid && $y eq $y_mid) {
    return "middle";
  } else {
    return "$x,$y";
  }
}
$Printwrap = 2;
my ($jx,$jy);
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = ($x+$dx, $y+$dy);

  my $chamfer = ($n==7||$n==8 || $n==10||$n==11 ? '\\MyJoin' : '\\MyChamfer');
  my $arrow;
  if ($n % 2 == 0) {
    $arrow = '->';
    $x2 .= chamfer(-$dx, $chamfer);
    $y2 .= chamfer(-$dy, $chamfer);
  } else {
    $arrow = '<-';
    $x .= chamfer($dx, $chamfer);
    $y .= chamfer($dy, $chamfer);
  }
  my $xy = xy_str($x,$y);
  my $xy2 = xy_str($x2,$y2);

  if (defined $jx) {
    printwrap(" \\draw ($jx,$jy) -- ($x,$y);");
    undef $jx;
  }
  printwrap(" \\draw[$arrow] ($xy) -- ($xy2);");

  if ($n % 2 == 0) {
    ($jx,$jy) = ($x2,$y2);
  }
}
printwrap_newline();

print "  % end generated\n";
exit 0;
