#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyPrintwrap ':all';


my $level = 4;
print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_mid) = $path->level_to_n_range($level);

my ($xm, $ym) = $path->n_to_xy($n_mid);
my $n_hi = 2*$n_mid;
print "  % $n_lo to $n_mid to $n_hi\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

$Printwrap = 2;
{
  my $n = $n_lo;
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = rotate($x,$y);
  printwrap("\\node (start) at ($x,$y) [my small dot];\n");
}
{
  my $n = $n_mid;
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = rotate($x,$y);
  printwrap("\\node (middle) at ($x,$y) [my small dot];\n");
}
{
  my $n = $n_hi;
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = rotate($x,$y);
  printwrap("\\node (end) at ($x,$y) [my small dot];\n");
}

sub rotate {
  my ($x,$y) = @_;
  my $rot4 = ($level + 1) / 2 % 4;
  foreach (1 .. $rot4) {
    ($x,$y) = xy_rotate_minus90($x,$y);
  }
  return ($x,$y);
}

sub xy_is_join {
  my ($x,$y) = @_;
  my @n_list = $path->xy_to_n_list($x,$y);
  return (@n_list > 1
          && $n_list[0] < $n_mid
          && $n_list[1] > $n_mid);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap = 4;
my $join = '';
my %seen;
my $n_join;
my $n_join_other;
my $n_join_dist;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($prev_dx,$prev_dy) = $path->n_to_dxdy($n-1);
  my @n_list = $path->xy_to_n_list($x,$y);
  my $is_join = xy_is_join($x,$y);

  ($x,$y) = rotate($x,$y);
  ($dx,$dy) = rotate($dx,$dy);
  if (defined $prev_dx) {
    ($prev_dx,$prev_dy) = rotate($prev_dx,$prev_dy);
  }

  if ($is_join) {
    $n_join //= $n;
    $n_join_other //= $n_list[1];
    $n_join_dist //= 2**$level - $n;

    my $sx = $x . chamfer(-$prev_dx);
    my $sy = $y . chamfer(-$prev_dy);
    my $tx = $x . chamfer($dx);
    my $ty = $y . chamfer($dy);
    printwrap("$join($sx,$sy)");
    printwrap("--($tx,$ty)");

    if ($n < $n_mid) {
      my ($rdx,$rdy) = xy_rotate_plus90($dx,$dy);
      my $x1 = $x;
      my $y1 = $y;
      my $x2 = $x + $dx + $rdx;
      my $y2 = $y + $dy + $rdy;
      ($x1,$y1, $x2,$y2) = rect_corners_sort ($x1,$y1, $x2,$y2);
      next if $seen{"$x1,$y1"}++;
    }

  } else {
    printwrap("$join($x,$y)");
  }
  $join = '--';
}
printwrap(";\n");

print "  \\newcommand{\\MyNmiddle}{$n_mid}\n";
print "  \\newcommand{\\MyJoinVertex}{$n_join}\n";
print "  \\newcommand{\\MyJoinVertexOther}{$n_join_other}\n";
print "  \\newcommand{\\MyJoinVertexDist}{$n_join_dist}\n";
print "% GP-Test-Last  JN($level)==$n_join\n";
print "% GP-Test-Last  JNother($level)==$n_join_other\n";
print "% GP-Test-Last  JND($level)==$n_join_dist\n";

print "  % end generated\n";
exit 0;
