#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'rect_corners_sort','xy_rotate_plus90';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 4;
print "  % generated by tools/$FindBin::Script\n";


sub path_n_is_left_boundary {
  my ($path, $n, $n_limit) = @_;
  ### path_n_is_left_boundary(): $n
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to side
  my $x1s = $x1 + $dx;
  my $y1s = $y1 + $dy;
  my $x2s = $x2 + $dx;
  my $y2s = $y2 + $dy;
  return (! path_xyxy_is_traversed_either($path, $x1,$y1, $x1s,$y1s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x2,$y2, $x2s,$y2s, $n_limit)
          || ! path_xyxy_is_traversed_either($path, $x1s,$y1s, $x2s,$y2s, $n_limit));
}
sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi - 1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

$Printwrap = 2;
printwrap("% N=$n_lo to $n_hi\n");
printwrap("\\node (start) at (0,0) [my dot];\n");
printwrap("\\node (end) at ($x_hi,$y_hi) [my dot];\n");

{
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");
}

my $squares;
my $length;
{
  $Printwrap = 2;
  printwrap("\\fill foreach \\p in {");
  $Printwrap = 4;
  my %seen;
  foreach my $n ($n_lo .. $n_hi - 1) {
    next unless path_n_is_left_boundary($path,$n, $n_hi);
    $length++;
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    my $x2 = $x1 + $dx;
    my $y2 = $y1 + $dy;
    ($dx,$dy) = xy_rotate_plus90($dx,$dy);  # to left side
    $x2 += $dx;
    $y2 += $dy;  # opposite corner

    ($x1,$y1, $x2,$y2) = rect_corners_sort ($x1,$y1, $x2,$y2);
    next if $seen{"$x1,$y1"}++;

    printwrap("($x1,$y1),");
    $squares++;
  }
  printwrap("}");
  printwrap(" { \\p +(\\MyMargin,\\MyMargin) rectangle +(1-\\MyMargin,1-\\MyMargin) };\n");
}

print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\newcommand{\\MySquares}{$squares}\n";
print "  \\newcommand{\\MyLength}{$length}\n";
print "% GP-Test-Last  LQ($level)==$squares\n";
print "% GP-Test-Last  L($level)==$length\n";

print "  % end generated\n";
exit 0;
