#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()
use Math::PlanePath::DragonMidpoint 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'chamfer','mul';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

{
  # dragon curve

  my $path = Math::PlanePath::DragonCurve->new;

  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  {
    my ($x,$y) = $path->n_to_xy($n_lo);
    print "  \\node (start) at ($x,$y) [my dot];\n";
  }
  {
    my ($x,$y) = $path->n_to_xy($n_hi);
    print "  \\node (end) at ($x,$y) [my dot];\n";
  }

  $Printwrap = 2;
  printwrap("\\draw[my line]\n");
  $Printwrap = 4;
  my $join = '';
  foreach my $n ($n_lo .. $n_hi) {
    if ($n == $n_lo) {
      printwrap("(start)");
    } elsif ($n == $n_hi) {
      printwrap("(end)");
    } else {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my ($prev_dx,$prev_dy) = $path->n_to_dxdy($n-1);
      my $sx = $x . chamfer(-$prev_dx);
      my $sy = $y . chamfer(-$prev_dy);
      my $tx = $x . chamfer($dx);
      my $ty = $y . chamfer($dy);
      printwrap("$join($sx,$sy)");
      printwrap("--($tx,$ty)");
    }
    $join = '--';
  }

  # {
  #   # end extension
  #   my $n = $n_hi;
  #   my ($dx,$dy) = $path->n_to_dxdy($n);
  #   printwrap("$join +($dx*\\MyEnd, $dy*\\MyEnd)");
  # }

  printwrap(";\n");
}

{
  # walls

  my $path = Math::PlanePath::DragonMidpoint->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    $segments->add($x,$y, $x+1,$y);
    $segments->add($x,$y+1, $x+1,$y+1);
    $segments->add($x,$y, $x,$y+1);
    $segments->add($x+1,$y, $x+1,$y+1);

    if ($n >= 1) {
      my ($dx,$dy) = $path->n_to_dxdy($n-1);
      if ($dx == 0) {
        if ($dy < 0) { $y += 1; }
        $segments->remove($x,$y, $x+1,$y);
      } else {
        if ($dx < 0) { $x += 1; }
        $segments->remove($x,$y, $x,$y+1);
      }
    }
  }

  # {
  #   # open end
  #   my $n = $n_hi + 1;
  #   my ($x,$y) = $path->n_to_xy($n);
  #   my ($dx,$dy) = $path->n_to_dxdy($n-1);
  #   if ($dx == 0) {
  #     if ($dy < 0) { $y += 1; }
  #     $segments->remove($x,$y, $x+1,$y);
  #   } else {
  #     if ($dx < 0) { $x += 1; }
  #     $segments->remove($x,$y, $x,$y+1);
  #   }
  # }

  my $transform = sub {
    my ($x,$y) = @_;
    ($x,$y) = mul($x,$y);
    return (($x+1)/2, ($y-1)/2);
  };

  $Printwrap = 2;
  printwrap("\\draw[my wall]");
  $Printwrap = 4;
  foreach my $line ($segments->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;
    ($x1,$y1) = $transform->($x1,$y1);
    ($x2,$y2) = $transform->($x2,$y2);
    printwrap(" ($x1,$y1)--($x2,$y2)");
  }
  printwrap(";\n");

  my @segs = $segments->segments;
  my $count = scalar(@segs);
  print "  \\newcommand{\\MyCount}{$count}\n";
  print "% GP-Test-Last  MW($level)==$count\n";
}

print "  % end generated\n";
exit 0;
