#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::DragonCurve->new;

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 2;
printwrap("\\node (start) at (0,0) [my dot];\n");
{
  my ($x,$y) = transform($x_hi,$y_hi);
  printwrap("\\node (end) at ($x,$y) [my dot];\n");
}

sub xy_is_double {
  my ($x,$y) = @_;
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ <= $n_hi} @n_list;
  return @n_list >= 2;
}

printwrap("\\draw[my line] ");
$Printwrap += 2;
my $join = '';
my $round = '[my sharp]';
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  my $want_round = (xy_is_double($x,$y) ? '[my sharp]' : '[my round]');
  ($x,$y) = transform($x,$y);

  if ($want_round ne $round) {
    $round = $want_round;
    printwrap($round);
  }

  my $p;
  if ($n == $n_lo) {
    $p = "(start)";
  } elsif ($n == $n_hi) {
    $p = "(end)";
  } else {
    $p = "($x,$y)";
  }

  my $semi = ($n == $n_hi ? ';' : '');
  printwrap("$join$p$semi");
  $join = '--';
}
printwrap_newline();

print "  % end generated\n";
exit 0;
