#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon ':all';
use MyPrintwrap ':all';


my $level = 4;  # of twindragon k=0 as square
print "    % generated by tools/$FindBin::Script\n";
print "    \\newcommand{\\MyLevel}{$level}\n";

{
  my $path4 = Math::PlanePath::DragonCurve->new (arms => 4);
  my ($n_lo, $n_hi) = $path4->level_to_n_range($level);

  sub xyxy_is_touch {
    my ($x,$y) = @_;
    my @n_list = $path4->xy_to_n_list($x,$y);
    @n_list = grep {$_ < $n_hi} @n_list;
    if (@n_list < 2) { return 0; }
    my @t_list = map {int($_/2) % 2} @n_list;
    return $t_list[0] != $t_list[1];
  }
}

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level+1);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 4;
printwrap("\\node (end-1) at ($x_hi,$y_hi) [my dot];\n");
{
  my $x = -$x_hi;
  my $y = -$y_hi;
  printwrap("\\node (end-2) at ($x,$y) [my dot];\n");
}

foreach my $negate (0, 1) {  # the two 180 degree copies
  $Printwrap = 4;
  printwrap("\\draw (origin)");
  $Printwrap = 6;
  foreach my $from_end (0, 1) {   # the two component k+1 dragons
    foreach my $n ($n_lo+1 .. $n_hi-$from_end) {
      my ($x,$y) = $path->n_to_xy($n);
      if ($from_end) {
        ($x,$y) = ($x_hi-$x, $y_hi-$y);
      }
      if ($negate) {
        $x = - $x;
        $y = - $y;
      }

      if (xyxy_is_touch($x,$y)) {
        my ($dx,$dy) = $path->n_to_dxdy($n);
        my ($prev_dx,$prev_dy) = $path->n_to_dxdy($n-1);

        if ($negate ^ $from_end) {
          $dx = - $dx;
          $dy = - $dy;
          $prev_dx = - $prev_dx;
          $prev_dy = - $prev_dy;
        }

        my $sx = $x . chamfer(-$prev_dx);
        my $sy = $y . chamfer(-$prev_dy);
        my $tx = $x . chamfer($dx);
        my $ty = $y . chamfer($dy);
        printwrap("--($sx,$sy)");
        printwrap("--($tx,$ty)");
      } else {
        printwrap("--($x,$y)");
      }
    }
  }
  printwrap("--(origin);\n");
}

print "    % end generated\n";
exit 0;
