#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyDragon 'transform_to_horizontal','dir';
use MyLineSegments;
use MyXYConvexHull;
use MyPrintwrap ':all';


my $level = 4;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

$Printwrap = 2;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y);
  my $dir = dir($n);
  if ($n&1) {
    ($x,$y) = $path->n_to_xy($n+1);
    $dir = -$dir;
  } else {
    ($x,$y) = $path->n_to_xy($n);
  }
  ($x,$y) = transform_to_horizontal($x,$y, $level);
  $dir -= $level/2;
  $dir = ($dir % 4) * 90;
  printwrap(" \\pic at ($x,$y) [transform shape,rotate=$dir] {my hull pic};");
}
printwrap_newline();

print "  % end generated\n";
exit 0;
