#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 118; # v.118 for xyxy_to_n_either()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'transform_to_horizontal','BlobN';
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";
my $level = 14;
if (@ARGV) { $level = shift @ARGV; }

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $scale = 2**($level/2);
print "  \\begin{scope}[scale=1/$scale]\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}

my $blob_segments = MyLineSegments->new;
my $other_segments = MyLineSegments->new;

my $blob_n_lo = BlobN($level);
my $blob_n_hi = BlobN($level + 1) - 3;

my $BlobSegments = 0;
my $BlobGSN_x = 0;  # total segment midpoint coordinates, *2
my $BlobGSN_y = 0;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  if ($n >= $blob_n_lo && $n < $blob_n_hi) {
    $blob_segments->add($x,$y, $x+$dx,$y+$dy);
    $BlobGSN_x += 2*$x + $dx;
    $BlobGSN_y += 2*$y + $dy;
    $BlobSegments++;
  } else {
    $other_segments->add($x,$y, $x+$dx,$y+$dy);
  }
}

$Printwrap = 2;
printwrap("\\draw[my segment colour] ");
$Printwrap += 2;
$blob_segments->printwrap (transform => \&transform);
printwrap(";");
printwrap(" % drawn with " . ($blob_segments->lines_joined_count_points) . " points\n");

$Printwrap = 2;
printwrap("\\draw[my other colour] ");
$Printwrap += 2;
$other_segments->printwrap (transform => \&transform);
printwrap(";");
printwrap(" % drawn with " . ($other_segments->lines_joined_count_points) . " points\n");

print "  \\end{scope}\n";
print "% GP-Test  BlobSegments($level) == $BlobSegments\n";
print "% GP-Test  BlobGSN($level) == $BlobGSN_x/2 + I*$BlobGSN_y/2\n";

print "  % end generated\n";
exit 0;
