#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;

use FindBin;
use lib $FindBin::Bin;
use MyPrintwrap ':all';


my $level = 8;

print "  % generated by tools/$FindBin::Script\n";

sub ffixedGap4pred {
  my ($n) = @_;
  my $ret = 1;
  while ($n%3 == 1) {
    $n = ($n-1)/3;
    $ret = 1-$ret;
  }
  if ($n % 3) {
    $ret = 1-$ret;
  }
  return $ret;
}
sub Gap4turn {
  my ($n) = @_;
  return 1 - 2*ffixedGap4pred($n);
}

our @dir3_to_dx = (2,-1,-1);
our @dir3_to_dy = (0, 1,-1);

$Printwrap = 2;
my ($x,$y) = (0,0);
my $d = 0;
my $segments = MyLineSegments->new;
my $count = 0;
my %seen;
foreach my $n (1 .. 3**$level) {
  my $dx = 2*$dir3_to_dx[$d % 3];
  my $dy = 2*$dir3_to_dy[$d % 3];
  my $x2 = $x + $dx;
  my $y2 = $y + $dy;
  die if $seen{"$x,$y,$x2,$y2"}++;
  $segments->add($x,$y, $x2,$y2);
  $count++;
  ($x,$y) = ($x2,$y2);
  $d += Gap4turn($n);
}
$count == 3**$level or die;
{
  # extra half segment
  my $dx = $dir3_to_dx[$d % 3];
  my $dy = $dir3_to_dy[$d % 3];
  my $x2 = $x + $dx;
  my $y2 = $y + $dy;
  $segments->add($x,$y, $x2,$y2);
  ($x,$y) = ($x2,$y2);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");
print "  \\coordinate (end) at ($x,$y) [my dot];\n";

print "% GP-Test  vector(100,n, Gap4turn(n)) == \\\n";
print "% GP-Test    [",join(',',map {Gap4turn($_)} 1 .. 100),"]\n";

print "  % end generated\n";

{
  # terdragon triangular

  package MyLineSegments;
  use 5.004;
  use strict;
  use Carp 'croak';
  use List::Util 'min','max','sum0','pairmap';
  use Scalar::Util 'looks_like_number';

  # uncomment this to run the ### lines
  # use Smart::Comments;

  sub new {
    my $class = shift;
    return bless { }, $class;
  }
  sub add {
    my ($self, $x1,$y1, $x2,$y2) = @_;
    ### add() ...

    if ($x1 > $x2) {
      ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
    }
    my $dy = $y2 - $y1;
    my $dx = $x2 - $x1;
    if ($dx == 0 && $dy == 0) {
      return;
    }
    unless ($dy == 0 || $dx==abs($dy)) {
      croak "Not triangular line: dx=$dx dy=$dy";
    }

    $dy /= $dx; # 1, 0, -1
    my $y = $y1 - $x1 * $dy;  # Y axis intersection
    ### $dy
    ### $y

    if (! defined $self->{'x_min'}) {
      $self->{'x_min'} = $self->{'x_max'} = $x1;
      $self->{'y_min'} = $self->{'y_max'} = $y;
    }
    $self->{'x_min'} = min($self->{'x_min'}, $x1, $x2);
    $self->{'x_max'} = max($self->{'x_max'}, $x1, $x2);
    $self->{'y_min'} = min($self->{'y_min'}, $y);
    $self->{'y_max'} = max($self->{'y_max'}, $y);

    foreach my $x ($x1 .. $x2-1) {
      $self->{$dy}->{$y}->{$x} = 1;
    }
  }
  sub remove {
    my ($self, $x1,$y1, $x2,$y2) = @_;
    if ($x1 > $x2) {
      ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
    }
    my $dy = $y2 - $y1;
    my $dx = $x2 - $x1;
    if ($dx == 0 && $dy == 0) {
      return;
    }
    unless ($dy == 0 || $dx==abs($dy)) {
      croak "Not triangular line: dx=$dx dy=$dy";
    }

    $dy /= $dx; # 1, 0, -1
    my $y = $y1 - $x1 * $dy;  # Y axis intersection

    foreach my $x ($x1 .. $x2-1) {
      delete $self->{$dy}->{$y}->{$x};
    }
  }
  sub remove_segments {
    my ($self, $segments) = @_;
    foreach my $seg ($segments->segments) {
      $self->remove(@$seg);
    }
  }

  # only meant for use on unit length segments
  sub xor {
    my ($self, $x1,$y1, $x2,$y2) = @_;
    if ($self->is_traversed($x1,$y1, $x2,$y2)) {
      $self->remove($x1,$y1, $x2,$y2);
    } else {
      $self->add($x1,$y1, $x2,$y2);
    }
  }

  sub is_traversed {
    my ($self, $x1,$y1, $x2,$y2) = @_;
    if ($x1 > $x2) {
      ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
    }
    my $dy = $y2 - $y1;
    my $dx = $x2 - $x1;

    $dy /= $dx; # 1, 0, -1
    my $y = $y1 - $x1 * $dy;  # Y axis intersection

    ### is_traversed(): "$x1,$y1 to $x2,$y2  dy=$dy"
    return (exists    $self->{$dy}
            && exists $self->{$dy}->{$y}
            && exists $self->{$dy}->{$y}->{$x1}
            &&        $self->{$dy}->{$y}->{$x1}
           );
  }
  sub lines {
    my ($self) = @_;
    ### lines(): $self
    my $x_min = $self->{'x_min'};
    my $x_max = $self->{'x_max'};
    my $y_min = $self->{'y_min'};
    my $y_max = $self->{'y_max'};
    my @ret;

    if (defined $x_min) {
      foreach my $dy (1, 0, -1) {
        foreach my $y_axis ($y_min .. $y_max) {
          my ($x_from,$x_to, $y_from,$y_to);
          my $x = $x_min;
          my $y = $y_axis + $dy*$x;
          for ( ; $x <= $x_max; $x++, $y += $dy) {
            # ### try: $x,$y, $x+1,$y+$dy
            my $x2 = $x + 1;
            my $y2 = $y + $dy;
            if ($self->is_traversed($x,$y, $x2,$y2)) {
              if (! defined $x_from) {
                ### new start ...
                $x_from = $x;
                $y_from = $y;
              }
              $x_to = $x2;
              $y_to = $y2;
            } else {
              if (defined $x_from) {
                push @ret, [ $x_from,$y_from, $x_to,$y_to ];
                ### push: [ $x_from,$y_from, $x_to,$y_to ]
                undef $x_from;
              }
            }
          }
        }
      }
    }
    ### $self
    return @ret;
  }

  sub segments {
    my ($self) = @_;
    ### lines(): $self
    my $x_min = $self->{'x_min'};
    my $x_max = $self->{'x_max'};
    my $y_min = $self->{'y_min'};
    my $y_max = $self->{'y_max'};
    my @ret;

    if (defined $x_min) {
      foreach my $dy (1, 0, -1) {
        foreach my $y_axis ($y_min .. $y_max) {
          my ($x_from,$x_to, $y_from,$y_to);
          my $x = $x_min;
          my $y = $y_axis + $dy*$x;
          for ( ; $x <= $x_max; $x++, $y += $dy) {
            # ### try: $x,$y, $x+1,$y+$dy
            my $x2 = $x + 1;
            my $y2 = $y + $dy;
            if ($self->is_traversed($x,$y, $x2,$y2)) {
              if ($dy == 0) {
                $x2++;
              }
              push @ret, [$x,$y, $x2,$y2];
              if ($dy == 0) {
                $x++;
              }
            }
          }
        }
      }
    }
    ### $self
    return @ret;
  }

  # Return a list of arrayrefs [$x1,$y1, $x2,$y2, ..., $xn,$yn] which are line
  # runs
  sub lines_joined {
    my ($self) = @_;
    my @lines = $self->lines;
    # print "separate lines ",scalar(@lines),"\n";

    my %start_hash;
    my %end_hash;
    my %all;

    my $add = sub {
      my ($line) = @_;
      ### add: $line
      {
        my $xy = "$line->[0],$line->[1]";
        push @{$start_hash{$xy}}, $line;
        $all{$xy} = 1;
      }
      {
        my $xy = "$line->[-2],$line->[-1]";
        push @{$end_hash{$xy}}, $line;
        $all{$xy} = 1;
      }
    };

    my $remove_from_aref = sub {
      my ($aref, $elem) = @_;
      my $length = @$aref;
      @$aref = grep { $_ != $elem } @$aref;
      if (@$aref != $length-1) {
        die "oops, bad remove";
      }
    };

    my $remove = sub {
      my ($line) = @_;
      ### remove: $line
      {
        my $xy = "$line->[0],$line->[1]";
        my $start_list = $start_hash{$xy};
        $remove_from_aref->($start_list, $line);
        ### remove to start_list: $start_list
      }
      {
        my $xy = "$line->[-2],$line->[-1]";
        my $end_list = $end_hash{$xy};
        $remove_from_aref->($end_list, $line);
        ### remove to start_list: $end_list
      }
    };

    foreach my $line (@lines) {
      $add->($line);
    }
    @lines = ();
    ### %start_hash
    ### %end_hash

    my @keys = keys %all;
    @keys = sort \&_xy_string_cmp, @keys;
    foreach my $xy (@keys) {
      ### $xy
      my $start_list = $start_hash{$xy} // [];
      my $end_list   = $end_hash{$xy}   // [];
      ### $start_list
      ### $end_list

      for (;;) {
        my $from_rev = 1;
        my $from_line = $start_list->[0]
          // do { $from_rev = 0; $end_list->[0] }
          // last;
        ### $from_rev
        ### $from_line
        $remove->($from_line);

        my $to_rev = 0;
        my $to_line = $start_list->[0]
          // do { $to_rev = 1; $end_list->[0] };
        if (! $to_line) {
          $add->($from_line);
          last;
        }
        ### $to_rev
        ### $to_line

        $remove->($to_line);

        if ($from_rev) {
          $from_line = _reverse_pairs($from_line);
        }
        if ($to_rev) {
          $to_line = _reverse_pairs($to_line);
        }

        ### join ...
        splice @$from_line, -2;
        my $new_line = [ @$from_line, @$to_line ];
        $add->($new_line);
      }
    }

    ### joins done ...

    foreach my $xy (@keys) {
      if (my $start_list = $start_hash{$xy}) {
        foreach my $line (@$start_list) {
          push @lines, $line;
          $remove->($line);
        }
      }
      if (my $end_list = $end_hash{$xy}) {
        foreach my $line (@$end_list) {
          push @lines, $line;
          $remove->($line);
        }
      }
    }

    # sort ends to go east and north
    @lines = map {
      my $line = $_;
      (($line->[0] <=> $line->[-2] || $line->[1] <=> $line->[-1]) <= 0)
        ? $line
        : _reverse_pairs($line)
      } @lines;

    $self->{'lines_joined_count_points'}
      = sum0(map {_lines_aref_count_points($_)} @lines);

    #  print "joined lines ",scalar(@lines),"\n";
    return @lines;
  }
  sub _lines_aref_count_points {
    my ($line) = @_;
    return scalar(@$line)/2 - ($line->[0] == $line->[-2]
                               && $line->[1] == $line->[-1]);
  }
  sub lines_joined_count_points {
    my ($self) = @_;
    if (! defined $self->{'lines_joined_count_points'}) {
      $self->lines_joined;
    }
    return $self->{'lines_joined_count_points'};
  }

  sub _reverse_pairs {
    my ($aref) = @_;
    my @ret = reverse @$aref;
    for (my $i = 0; $i <= $#ret; $i += 2) {
      ($ret[$i], $ret[$i+1]) = ($ret[$i+1], $ret[$i]);
    }
    return \@ret;
  }

  sub bbox {
    my ($self) = @_;
    # ENHANCE-ME: this is not efficient.
    my @segments = $self->segments or return;
    my @x = map {($_->[0],$_->[2])} @segments;
    my @y = map {($_->[1],$_->[3])} @segments;
    return (min(@x),min(@y), max(@x),max(@y));
  }

  #------------------------------------------------------------------------------
  # printwrap bits

  sub _transform_noop {
    return @_;
  }
  sub printwrap {
    my ($segments, %options) = @_;
    my $transform = $options{'transform'} || \&_transform_noop;
    require MyPrintwrap;
    my $separator = '';
    foreach my $line ($segments->lines_joined) {
      my @points;
      my ($prev_x,$prev_y);
      for (my $i = 0; $i < $#$line; $i+=2) {
        my $x = $line->[$i];
        my $y = $line->[$i+1];
        if ($i == $#$line-1 && $x == $line->[0] && $y == $line->[1]) {
          push @points, 'cycle';
          next;
        }
        ($x,$y) = $transform->($x,$y);
        my $str = "($x,$y)";
        if (defined $prev_x
            && looks_like_number($x)
            && looks_like_number($y)
            && looks_like_number($prev_x)
            && looks_like_number($prev_y)) {
          my $dx = $x - $prev_x;
          my $dy = $y - $prev_y;
          my $dstr = "+($dx,$dy)";
          if (CORE::length($dstr) < CORE::length($str)) {
            push @points, $dstr;
            next; # $prev_x,$prev_y unmoved
          }
        }
        $prev_x = $x;
        $prev_y = $y;
        push @points, $str;
      }

      # my @points = pairmap {
      #   my ($x,$y) = $transform->($a,$b);
      #   "($x,$y)"
      # } @$line;
      # if ($points[-1] eq $points[0]) {
      #   $points[-1] = 'cycle';
      # }
      my $join = $separator;
      foreach my $p (@points) {
        MyPrintwrap::printwrap("$join$p");
        $join = '--';
      }
      $separator = ' ';
    }
  }
}
exit 0;
