#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon ':all';
use MyPrintwrap ':all';


my $levels = 8;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::DragonCurve->new;

sub is_touch {
  my ($n, $n_hi) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  foreach my $n_other ($path->xy_to_n_list($x,$y)) {
    next if $n_other == $n;
    next if $n_other > $n_hi;
    return 1;
  }
  return 0;
}

foreach my $k (4 .. $levels) {
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$k)]\n");

  my $n_lo = BlobN($k) + 2;
  my $n_hi = BlobN($k+1) + 1;

  {
    my ($x_lo,$y_lo) = $path->n_to_xy($n_lo+.5);
    printwrap("\\coordinate (start-$k) at ($x_lo,$y_lo);\n");
  }
  {
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi-.5);
    printwrap("\\coordinate (end-$k) at ($x_hi,$y_hi);\n");
  }
  # printwrap("\\node at ($x_lo,$y_lo) [my dot,name=start-$k];\n");
  # printwrap("\\node at ($x_hi,$y_hi) [my dot,name=end-$k];\n");

  $Printwrap = 4;
  printwrap("\\draw[my line] ");

  my $join = '';
  foreach my $n ($n_lo .. $n_hi) {
    if ($n == $n_lo) {
      my ($x,$y) = $path->n_to_xy($n+.5);
      printwrap("$join($x,$y)");
    } elsif ($n == $n_hi) {
      my ($x,$y) = $path->n_to_xy($n-.5);
      printwrap("$join($x,$y)");
    } else {
      my ($x,$y) = $path->n_to_xy($n);
      my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
      my ($tdx,$tdy) = $path->n_to_dxdy($n);
      my $sx = $x . chamfer(-$sdx);
      my $sy = $y . chamfer(-$sdy);
      my $tx = $x . chamfer($tdx);
      my $ty = $y . chamfer($tdy);
      printwrap("--($sx,$sy)");
      printwrap("--($tx,$ty)");
    }
    $join = '--';
  }
  printwrap(";\n");
  $Printwrap = 2;
  printwrap("\\end{scope}\n");
}

print "  % end generated\n";
exit 0;
