#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::DragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyDragon 'BlobN','xy_rotate_plus90','xy_rotate_minus90','chamfer';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::DragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
my $y_min = -13;

$Printwrap = 2;
printwrap("\\node (start) at (0,0) [my dot];\n");
printwrap("\\node (end) at ($x_hi,$y_hi) [my dot];\n");

# above
sub above_transform {
  my ($x,$y) = @_;
  return xy_rotate_plus90($x,$y);
}
sub above_untransform {
  my ($x,$y) = @_;
  return xy_rotate_minus90($x,$y);
}
my $above_n_end = BlobN($level);

# below
sub below_transform {
  my ($x,$y) = @_;
  return xy_rotate_minus90($x,$y);
}
sub below_untransform {
  my ($x,$y) = @_;
  return xy_rotate_plus90($x,$y);
}
my $below_n_start = BlobN($level)-3;
my $below_n_end   = 2*2**$level - BlobN($level) + 1;

sub is_touch {
  my ($x,$y) = @_;
  my @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ < $n_hi} @n_list;
  if (@n_list >= 2) { return 0; }

  {
    my ($x,$y) = above_untransform($x,$y);
    @n_list = $path->xy_to_n_list($x,$y);
    @n_list = grep {$_ < $n_hi} @n_list;
    if (@n_list == 1) { return 1; }
  }

  ($x,$y) = below_untransform($x,$y);
  @n_list = $path->xy_to_n_list($x,$y);
  @n_list = grep {$_ < 2*$n_hi} @n_list;
  return (@n_list == 1);
}

sub draw {
  my ($n_start,$n_end, $transform, $n_subcurve) = @_;
  my $join = '';
  foreach my $n ($n_start .. $n_end-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);

    ($x,$y)   = $transform->($x,$y);
    ($x2,$y2) = $transform->($x2,$y2);

    if ($y < $y_min && $y2 < $y_min) { next; }
    if ($y < $y_min) { $join = ''; }

    my $dx = $x2 - $x;
    my $dy = $y2 - $y;

    my $xy_touch = is_touch($x,$y);
    my $x2y2_touch = is_touch($x2,$y2);

    if ($xy_touch) {
      $x .= chamfer($dx);
      $y .= chamfer($dy);
      printwrap("$join($x,$y)");
      $join = '--';
    } elsif ($join eq '') {
      printwrap("$join($x,$y)");
      $join = '--';
    }
    if ($x2y2_touch) {
      $x2 .= chamfer(-$dx);
      $y2 .= chamfer(-$dy);
    }
    printwrap("$join($x2,$y2)");
    $join = '--';
  }
}

$Printwrap = 2;
draw($n_lo, $n_hi, sub { @_ });
draw($n_lo,          $above_n_end, \&above_transform);
draw($below_n_start, $below_n_end, \&below_transform);
printwrap(";\n");

{
  my ($x,$y) = $path->n_to_xy($above_n_end);
  ($x,$y) = above_transform($x,$y);
  print "  \\coordinate (above-end) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($below_n_start);
  ($x,$y) = below_transform($x,$y);
  print "  \\coordinate (below-start) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($below_n_end);
  ($x,$y) = below_transform($x,$y);
  print "  \\coordinate (below-end) at ($x,$y);\n";
}

print "  % end generated\n";
exit 0;
